// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <cstddef>
#include <iostream>
#include <string>

#include <azure-device-util/StreamCopier.h>

#include "CrashParameters.h"

namespace Azure::Device::CrashHandler {

struct CrashAnalysis
{
    // Wer report parameters
    std::string AppName = "unknown";
    std::string AppVersion = "0";
    std::string PackageName = "unknown";
    std::string PackageVersion = "unknown";
    std::string PackageArch = "unknown";
    std::string ModuleName = "unknown";
    std::string ModuleVersion = "0";
    uint64_t ModuleOffset = 0;
    uint64_t ExceptionCode = 0;
    uint64_t StackHash = 0;

    bool Is32Bits = false;

    friend std::ostream&
    operator<<(std::ostream& os, const CrashAnalysis& analysis)
    {
        os << "[CrashAnalysis]" << std::endl;
        os << "AppName: " << analysis.AppName << std::endl;
        os << "AppVersion: " << analysis.AppVersion << std::endl;
        os << "PackageName: " << analysis.PackageName << std::endl;
        os << "PackageVersion: " << analysis.PackageVersion << std::endl;
        os << "PackageArch: " << analysis.PackageArch << std::endl;
        os << "ModuleName: " << analysis.ModuleName << std::endl;
        os << "ModuleVersion: " << analysis.ModuleVersion << std::endl;
        os << "ModuleOffset: 0x" << std::hex << analysis.ModuleOffset
           << std::endl;
        os << "ExceptionCode: " << std::dec << analysis.ExceptionCode
           << std::endl;
        os << "StackHash: 0x" << std::hex << analysis.StackHash << std::endl;
        return os;
    }
};

CrashAnalysis CopyAndAnalyzeCrash(
    const CrashParameters& params,
    Azure::Device::Utils::StreamCopier& coreStreamCopier,
    std::ostream& logStream);
}
