// Copyright (c) Microsoft Corporation. All rights reserved.

#include "CrashSubmission.h"

#include <sstream>

#include <azure-device-errorreporting-submitter/ReportQueue.h>

#include "Config.h"

namespace Azure::Device::CrashHandler {

std::string GetWatsonReportName()
{
    return AZURE_DEVICE_WATSON_REPORT_NAME;
}

uint64_t SubmitCrashToUploaderService(
    const std::filesystem::path& queuePath,
    const CrashAnalysis& analysis,
    const std::vector<std::filesystem::path>& pathsToAddToReport)
{
    auto queue =
        Azure::Device::ErrorReporting::Submitter::ReportQueue::OpenReportQueue(
            queuePath);

    auto report = queue->CreateReport(GetWatsonReportName());

    // Register file path for logs
    report->AddBucketingParameter(analysis.AppName);
    report->AddBucketingParameter(analysis.AppVersion);
    report->AddBucketingParameter(analysis.ModuleName);
    report->AddBucketingParameter(analysis.ModuleVersion);

    auto addHexParameter = [&report](uint64_t param) {
        std::stringstream hexFormat;
        hexFormat << std::hex << param;
        report->AddBucketingParameter(hexFormat.str());
    };

    addHexParameter(analysis.ModuleOffset);
    addHexParameter(analysis.ExceptionCode);
    addHexParameter(analysis.StackHash);

    for (const auto& path : pathsToAddToReport)
    {
        report->RegisterPath(path);
    }

    queue->Submit(*report);

    return report->GetId();
}
}
