// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <string>
#include <string_view>
#include <vector>

#include "azure-device-util/Guid.h"

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Submitter {

class Report
{
  public:
    Report() = delete;

    Report(std::string_view eventType);

    const std::string& GetEventType() { return m_eventType; }
    const std::vector<std::string>& GetBucketingParameters()
    {
        return m_bucketingParameters;
    }
    const std::vector<std::pair<std::filesystem::path, std::filesystem::path>>&
    GetRegisteredPaths()
    {
        return m_registeredPaths;
    }
    uint64_t GetId() { return m_reportId; }

    void AddBucketingParameter(std::string_view parameter);

    void RegisterPath(const std::filesystem::path& sourcePath);

    void RegisterPathInSubdirectory(
        const std::filesystem::path& sourcePath,
        const std::filesystem::path& reportSubdirPath);

    void WriteMetadataAndEmitLogging(std::filesystem::path& metadataPath);

  private:
    std::string m_eventType;
    std::vector<std::string> m_bucketingParameters;
    std::vector<std::pair<std::filesystem::path, std::filesystem::path>> m_registeredPaths;
    uint64_t m_reportId = 0;
    uuid_t m_werReportId;
};

}}}}
