// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <fstream>

#include <zipper/zipper.h>

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Submitter { namespace ZipUtils {

/*++
    Adds a file or directory path to a zip file.

Arguments:
    zipper - The Zipper instance that is building the final zip file.
    sourcePath - The path to the file or directory to add to the zip file.
    zipSubdirPath - A path that specifies where sourcePath should be placed in
the zip file. An empty string will place sourcePath at the root of the zip file.
--*/
static void AddPathToZip(
    zipper::Zipper& zipper,
    const std::filesystem::path& sourcePath,
    const std::filesystem::path& zipSubdirPath)
{
    if (std::filesystem::is_directory(sourcePath))
    {
        for (auto& item : std::filesystem::directory_iterator(sourcePath))
        {
            AddPathToZip(
                zipper,
                item.path(),
                std::filesystem::is_directory(item.path()) ?
                    zipSubdirPath / item.path().filename() :
                    zipSubdirPath);
        }
    }
    else
    {
        std::filesystem::path filePathInZip =
            zipSubdirPath / sourcePath.filename();
        std::ifstream inputStream(sourcePath.string().c_str(), std::ios::binary);
        zipper.add(inputStream, filePathInZip.string());
    }
}

}}}}}
