// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <atomic>
#include <string>
#include <vector>

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Uploader {

struct INotifyMonitorEvent
{
    uint32_t Mask;
    uint32_t Cookie;
    std::string Name;

    INotifyMonitorEvent() = default;

    INotifyMonitorEvent(uint32_t mask, uint32_t cookie, const char* name)
        : Mask(mask), Cookie(cookie), Name(name)
    {}
};

class INotifyMonitorCallbackReceiver
{
  public:
    virtual ~INotifyMonitorCallbackReceiver() {}

    virtual void
    INotifyMonitorCallback(std::vector<INotifyMonitorEvent> events) = 0;
};

class INotifyMonitor
{
  public:
    INotifyMonitor(INotifyMonitorCallbackReceiver* callbackReceiver);

    ~INotifyMonitor();

    void AddMonitoredPath(const std::string& path);

    void Run();

    void Stop();

    void SetPollIntervalSeconds(time_t interval)
    {
        m_pollIntervalSeconds = interval;
    }

  public:
    static const time_t k_defaultPollIntervalSeconds = 60 * 15;

  private:
    INotifyMonitorCallbackReceiver* m_callbackReceiver;
    std::vector<std::string> m_paths;
    std::vector<int> m_watchDescriptors;
    int m_monitorDescriptor = -1;
    std::atomic_bool m_isStopping;
    int m_stopDescriptor = -1;
    time_t m_pollIntervalSeconds = k_defaultPollIntervalSeconds;

    static const size_t k_maxReadBufferSize = 1024;
};

}}}}
