// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <memory>
#include <string_view>

#include "ErrorReportUploadServiceBase.h"

#if defined(AZURE_DEVICE_ERRORREPORTING_UPLOADER_USE_HTTP_UPLOADER) || \
    defined(AZURE_DEVICE_ERRORREPORTING_UPLOADER_USE_BONDI_UPLOADER)
    #include "LocalPolicyManager.h"
#endif

#ifdef AZURE_DEVICE_ERRORREPORTING_UPLOADER_USE_HVSOCKET_UPLOADER
    #include "RemotePolicyManager.h"
#endif

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Uploader {

class ErrorReportUploadServiceFactory
{
  public:
#ifdef AZURE_DEVICE_ERRORREPORTING_UPLOADER_USE_HTTP_UPLOADER
    static std::unique_ptr<ErrorReportUploadServiceBase>
    CreateHttpErrorReportUploadService(
        Policy::LocalPolicyManager& localPolicyManager);
#endif

#ifdef AZURE_DEVICE_ERRORREPORTING_UPLOADER_USE_BONDI_UPLOADER
    static std::unique_ptr<ErrorReportUploadServiceBase>
    CreateBondiErrorReportUploadService(
        Policy::LocalPolicyManager& localPolicyManager);
#endif

#ifdef AZURE_DEVICE_ERRORREPORTING_UPLOADER_USE_HVSOCKET_UPLOADER
    static std::unique_ptr<ErrorReportUploadServiceBase>
    CreateHvSocketErrorReportUploadService(
        Policy::RemotePolicyManager& remotePolicyManager);
#endif
};

}}}}
