// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <string>
#include <string_view>

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Uploader {

class ProtocolArgumentSet
{
  public:
    ProtocolArgumentSet() {}

    const std::map<std::string, std::string, std::less<>>&
    GetProtocolArgsForSection(std::string_view section)
    {
        return m_protocolArgs[std::string{ section }];
    }

    bool IsProtocolArgSet(std::string_view section, std::string_view name)
    {
        auto sectionItr = m_protocolArgs.find(section);
        if (sectionItr == m_protocolArgs.end())
        {
            return false;
        }

        return sectionItr->second.find(name) != sectionItr->second.end();
    }

    std::string GetProtocolArg(std::string_view section, std::string_view name)
    {
        auto sectionItr = m_protocolArgs.find(section);
        if (sectionItr == m_protocolArgs.end())
        {
            return "";
        }

        auto argItr = sectionItr->second.find(name);

        return argItr == sectionItr->second.end() ? "" : argItr->second;
    }

    void AddProtocolArg(
        std::string_view section,
        std::string_view name,
        std::string_view value)
    {
        m_protocolArgs[std::string{ section }].emplace(name, value);
    }

  private:
    std::map<std::string, std::map<std::string, std::string, std::less<>>, std::less<>>
        m_protocolArgs;
};
}}}}
