// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <memory>
#include <pugixml.hpp>
#include <string>

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Uploader {

class ProtocolArgumentSet;

class ProtocolXmlDocument
{
  public:
    ProtocolXmlDocument(std::shared_ptr<ProtocolArgumentSet> protocolArgs);

    void SetNamespace(
        const std::string& svc,
        const std::string& partner,
        const std::string& group,
        const std::string& app);

    void AddCommand(
        const std::string& name,
        const std::map<std::string, std::string>& args);

    void AddPayload(const std::string& size);

    std::string ToProtocolString();

  private:
    void Initialize();

    void
    AddProtocolSection(const std::string& sectionName, pugi::xml_node parentNode);

  private:
    pugi::xml_document m_xmlDoc;
    std::shared_ptr<ProtocolArgumentSet> m_protocolArgs;

    pugi::xml_node m_reqsNode;
    pugi::xml_node m_reqNode;
};

}}}}
