// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <pugixml.hpp>
#include <stdexcept>

class xml_parse_error_category_t : public std::error_category
{
    const char* name() const noexcept override { return "XmlParseError"; }

    std::string message(int ev) const override
    {
        pugi::xml_parse_result result;
        result.status = static_cast<pugi::xml_parse_status>(ev);
        return result.description();
    }
};

xml_parse_error_category_t xml_parse_error_category;

std::system_error make_xml_parse_error(pugi::xml_parse_result result)
{
    std::error_code e(static_cast<int>(result.status), xml_parse_error_category);
    return std::system_error(e);
}
