// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <cpprest/http_client.h>

#include "ProtocolArgumentSet.h"

namespace Azure { namespace Device { namespace ErrorReporting {
namespace Uploader {

class HttpReportUploader
{
  public:
    HttpReportUploader(const std::filesystem::path& reportPath);

    bool BucketHitReceived() { return !m_bucket.empty(); }
    bool CabRequested()
    {
        return BucketHitReceived() && !m_buckettbl.empty() && !m_token.empty();
    }
    bool CabReceived() { return !m_cabId.empty(); }
    std::string& GetBucket() { return m_bucket; }
    std::string& GetBucketTable() { return m_buckettbl; }
    std::string& GetToken() { return m_token; }
    std::string& GetCabId() { return m_cabId; }
    std::string& GetEventType() { return m_eventType; }
    std::vector<std::string>& GetBucketingParameters()
    {
        return m_bucketingParameters;
    }
    uint64_t GetReportId() { return m_reportId; }

    bool SetProxy(std::string_view proxy) 
    { 
        if (web::uri::validate(std::string{proxy}) && !proxy.empty())
        {
            m_proxy = proxy;
            return true;
        }
        
        return false;
    }
    void SetValidateCertificates(bool enable)
    {
        m_validateCertificates = enable;
    }

    void AddProtocolArg(
        std::string_view section,
        std::string_view name,
        std::string_view value);

    enum class UploadType
    {
        BucketHitOnly,
        DumpUpload
    };
    void Upload(UploadType uploadType);

  private:
    std::string CreateEventRequestMessage();

    std::string CreateDataUploadRequestMessage(size_t size);

    void ParseEventResponse(web::http::http_response httpResponse);

    void ParseDataUploadResponse(web::http::http_response httpResponse);

    void ParseMetadata();

  private:
    std::filesystem::path m_reportPath;
    std::string m_eventType;
    std::vector<std::string> m_bucketingParameters;
    std::shared_ptr<ProtocolArgumentSet> m_protocolArgs;
    std::string m_bucket;
    std::string m_buckettbl;
    std::string m_token;
    std::string m_cabId;
    std::string m_proxy;
    uint64_t m_reportId = 0;
    bool m_validateCertificates = true;
};
}}}}
