// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <optional>
#include <string>
#include <vector>

// Architecture IDs
// From:
// https://docs.microsoft.com/en-us/windows/win32/api/sysinfoapi/ns-sysinfoapi-system_info
enum class ProcessArchitectureId
{
    UNKNOWN = 0xffff,
    INTEL = 0,   // x86
    AMD64 = 9,   // x64 (AMD or Intel)
    ARM = 5,     // ARM
    ARM64 = 12,  // ARM64
    IA64 = 6     // Intel Itanium-based
};

struct CpuInfo
{
    uint32_t NumCpus = 1;  // Number of logical CPU units. (Threads per core X
                           // cores per socket X Sockets)
    uint32_t Sockets = 1;  //
    uint32_t CorePerSocket = 0;  // Physical core per CPU

    // Architecture Id as defined in Windows
    ProcessArchitectureId ArchitectureId = ProcessArchitectureId::UNKNOWN;

    double CpuClockSpeed = 0.0;  // Frequency/speed of the processor
    std::string Arch;            // CPU architecture
    std::string CpuVendor;       // CPU vendor
    std::string CpuModel;        // CPU model

    std::string HypervisorVendor;  // HyperVisor vendor (if available)
};

namespace Azure { namespace Device { namespace Health {

class CpuInfoParser
{
  public:
    std::optional<CpuInfo> Parse();

  private:
    std::vector<std::string> GetLscpuOutput();
    std::optional<CpuInfo> ParseLines(const std::vector<std::string>& lscpuLines);
};

}}}
