// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <vector>

#include <azure-device-health-plugin-utils/IProcessMetadataCache.h>
#include <azure-device-util/ResultException.h>

#include "CpuProcessMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class PerProcessMetricStorage
{
  public:
    struct ReportMetrics
    {
        uint32_t ProcessId;
        std::string ProcessName;
        long long ClockTicksOverPeriod;
    };

    struct UserAndSystemMetrics
    {
        int32_t TotalWindowMs;
        std::vector<ReportMetrics> UserMetrics;
        std::vector<ReportMetrics> SystemMetrics;
    };

    void
    AddMetric(uint64_t sampleKey, const PerProcessCpuSample& perProcessCpuSample);

    bool IsReportReady()
    {
        if (static_cast<int32_t>(_ringMetrics.size()) < _reportCadence)
        {
            // Configuration error, this will never succeed in reporting
            FAIL_FAST();
        }

        return (_samplesSinceLastReport >= _reportCadence);
    }
    UserAndSystemMetrics GenerateReportMetrics(uint32_t reportProcessCount);

    void FireReportMetrics(
        Azure::Device::Health::PluginUtils::IProcessMetadataCache& processMetadataCache,
        uint64_t sampleKey,
        uint64_t clockTicksPerSecond,
        const UserAndSystemMetrics& metrics) const;

    void SetBufferSize(int64_t size) { _ringMetrics.resize(size); }
    void SetReportCadence(int32_t reportCadence)
    {
        _reportCadence = reportCadence;
    }

  private:
    std::vector<PerProcessMetricStorage::ReportMetrics>
    GenerateReportMetricsHelper(
        uint32_t reportProcessCount,
        const std::pair<unsigned long, std::vector<CpuProcessInfo>>& oldestSample,
        const std::pair<unsigned long, std::vector<CpuProcessInfo>>& newestSample,
        CpuProcessState sampleIndex) const;

    void IncrementWithWrap(int32_t& value, int32_t wrapSize)
    {
        value++;
        if (value >= wrapSize)
        {
            value = 0;
        }
    }

  private:
    std::vector<std::pair<unsigned long, std::vector<CpuProcessInfo>>> _ringMetrics;
    int32_t _currentRingEntry = 0;
    int32_t _oldestReportedEntry = -1;
    int32_t _newestValidEntry = -1;

    int32_t _samplesSinceLastReport = 0;
    int32_t _reportCadence = 0;
};

}}}}
