// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <vector>

#include <azure-device-health-plugin/plugin-dev.h>
#include <azure-device-util/ResultException.h>

#include "CpuMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class CpuMetricStorage
{
  public:
    struct ReportMetrics
    {
        int64_t TotalWindowDurationMs;
        uint32_t AverageTimePerSampleMs;
        uint64_t ContextSwitchesInWindow;
        int64_t TotalUserTicks;
        int64_t TotalNiceTicks;
        int64_t TotalSystemTicks;
        int64_t TotalIoTicks;
        int64_t TotalIrqTicks;
        int64_t TotalSoftIrqTicks;
        int64_t TotalStealTicks;
        int64_t TotalGuestTicks;
        int64_t TotalGuestNiceTicks;
        int32_t DecileCounts[10];
    };

    void AddMetric(uint64_t sampleKey, const TotalCpuSample& sample);

    bool IsReportReady()
    {
        if (static_cast<int32_t>(_ringMetrics.size()) < _reportCadence)
        {
            // Configuration error, this will never succeed in reporting
            FAIL_FAST();
        }

        return (_samplesSinceLastReport >= _reportCadence);
    }

    ReportMetrics GenerateReportMetrics();

    void
    FireReportMetrics(uint64_t clockTicksPerSecond, const ReportMetrics& metrics);

    void SetBufferSize(int64_t size) { _ringMetrics.resize(size); }
    void SetReportCadence(int32_t reportCadence)
    {
        _reportCadence = reportCadence;
    }

  private:
    void IncrementWithWrap(int32_t& value, int32_t wrapSize)
    {
        value++;
        if (value >= wrapSize)
        {
            value = 0;
        }
    }

  private:
    std::vector<std::pair<uint64_t, CpuInfo>> _ringMetrics;
    int32_t _currentRingEntry = 0;
    int32_t _oldestReportedEntry = -1;
    int32_t _newestValidEntry = -1;

    int32_t _samplesSinceLastReport = 0;
    int32_t _reportCadence = 0;
};

}}}}
