// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <string>
#include <vector>

#include <azure-device-util/FileSystem.h>
#include <azure-device-util/String.h>

#include "CpuMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class ProcStatParser
{
  public:
    ProcStatParser() : _rootPath("/") {}

    ProcStatParser(std::string_view rootPath) : _rootPath(rootPath) {}

    bool Parse(CpuInfo& info, bool summaryMode)
    {
        std::string cpuInfoPath;
        cpuInfoPath.append(_rootPath);
        cpuInfoPath.append("proc/stat");
        std::string contents;
        if (FAILED(Utils::FileSystem::ReadUTF8StringFromFile(
                cpuInfoPath, contents)))
        {
            return false;
        }

        std::vector<std::string> lines = Utils::String::Split(contents, "\n");
        for (const auto& line : lines)
        {
            if (Utils::String::IStartsWith(line, "cpu"))
            {
                constexpr uint8_t idIndex = 3;
                char cpuId = line[idIndex];
                if (cpuId == ' ')
                {
                    std::vector<std::string> counters =
                        Utils::String::Split(&(line[idIndex + 2]), " ");

                    if (summaryMode)
                    {
                        uint32_t numCounters = std::min(
                            static_cast<uint32_t>(counters.size()),
                            static_cast<uint32_t>(CpuState::NUM_VALUES));
                        for (size_t x = 0; x < numCounters; x++)
                        {
                            info.Sample[x] = atoll(counters[x].c_str());
                        }
                    }
                    else
                    {
                        // TODO: Support breakdown by CPU
                        return false;
                    }
                }
            }
            else if (Utils::String::IStartsWith(line, "ctxt"))
            {
                info.ContextSwitches = atoll(line.c_str() + 4);
            }
        }

        return true;
    }

  private:
    std::string _rootPath;
};

}}}}
