// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <string>
#include <vector>

#include <azure-device-util/FileSystem.h>
#include <azure-device-util/String.h>

#include "DiskMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class DiskStatParser
{
  public:
    DiskStatParser() : _rootPath("/") {}

    DiskStatParser(std::string_view rootPath) : _rootPath(rootPath) {}

    bool Parse(TotalDiskIoSample& finalInfo, bool summaryMode)
    {
        std::string diskInfoPath;
        diskInfoPath.append(_rootPath);
        diskInfoPath.append("proc/diskstats");

        std::string contents;
        if (FAILED(Utils::FileSystem::ReadUTF8StringFromFile(
                diskInfoPath, contents)))
        {
            return false;
        }

        std::vector<std::string> lines = Utils::String::Split(contents, "\n");

        for (auto& line : lines)
        {
            if (line.empty() || line.length() == 0)
            {
                continue;
            }
            Utils::String::Trim(line);
            std::vector<std::string> counters =
                Utils::String::Split(line, " ", false);

            if (Utils::String::IStartsWith(counters[2], "sda") ||
                Utils::String::IStartsWith(counters[2], "hda"))
            {
                DiskIoInfo info;
                info.DeviceName = counters[2];
                if (summaryMode && counters[2].length() == 3)
                {
                    info.DeviceName = counters[2];
                    uint32_t numCounters = std::min(
                        static_cast<uint32_t>(counters.size()),
                        static_cast<uint32_t>(DiskStats::NUM_VALUES));
                    for (size_t x = 0; x < numCounters; x++)
                    {
                        if (x == DiskStats::DEVICE_NAME)
                        {
                            continue;
                        }
                        info.Sample[x] = atoll(counters[x].c_str());
                    }
                    finalInfo.Disks.push_back(info);
                }
                else if (!summaryMode)
                {
                    info.DeviceName = counters[2];
                    uint32_t numCounters = std::min(
                        static_cast<uint32_t>(counters.size()),
                        static_cast<uint32_t>(DiskStats::NUM_VALUES));
                    for (size_t x = 0; x < numCounters; x++)
                    {
                        if (x == DiskStats::DEVICE_NAME)
                        {
                            continue;
                        }
                        info.Sample[x] = atoll(counters[x].c_str());
                    }
                    finalInfo.Disks.push_back(info);
                }
            }
        }

        return true;
    }

  private:
    std::string _rootPath;
};

}}}}
