// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <vector>

#include <azure-device-util/ResultException.h>

#include "MemoryMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class MemoryMetricStorage
{
  public:
    struct ReportMetrics
    {
        uint32_t TotalWindowDurationMs;
        uint32_t MemTotalKb;
        uint32_t MemFreeKb;
        uint32_t BuffersKb;
        uint32_t CachedKb;
        uint32_t SwapCachedKb;
        uint32_t ActiveKb;
        uint32_t InactiveKb;
        uint32_t HighTotalKb;
        uint32_t HighFreeKb;
        uint32_t LowTotalKb;
        uint32_t LowFreeKb;
        uint32_t SwapTotalKb;
        uint32_t SwapFreeKb;
        uint32_t DirtyKb;
        uint32_t WritebackKb;
        uint32_t AnonPagesKb;
        uint32_t MappedKb;
        uint32_t SharedMemoryKb;
        uint32_t CommitLimitKb;
        uint32_t CommitedAsKb;
        uint32_t VmallocTotalKb;
        int32_t TotalMemoryDecileCounts[10];
    };

    void AddMetric(uint64_t sampleKey, const TotalMemorySample& sample);

    bool IsReportReady()
    {
        if (static_cast<int32_t>(_ringMetrics.size()) < _reportCadence)
        {
            // Configuration error, this will never succeed in reporting
            FAIL_FAST();
        }

        return (_samplesSinceLastReport >= _reportCadence);
    }

    ReportMetrics GenerateReportMetrics();

    void FireReportMetrics(const ReportMetrics& metrics);

    void SetBufferSize(int64_t size) { _ringMetrics.resize(size); }
    void SetReportCadence(int32_t reportCadence)
    {
        _reportCadence = reportCadence;
    }

  private:
    void IncrementWithWrap(int32_t& value, int32_t wrapSize)
    {
        value++;
        if (value >= wrapSize)
        {
            value = 0;
        }
    }

  private:
    std::vector<std::pair<uint64_t, TotalMemorySample>> _ringMetrics;
    int32_t _currentRingEntry = 0;
    int32_t _oldestValidEntry = -1;
    int32_t _newestValidEntry = -1;

    int32_t _samplesSinceLastReport = 0;
    int32_t _reportCadence = 0;
};

}}}}
