// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <vector>

#include <azure-device-util/ResultException.h>

#include "NetworkMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class NetworkMetricStorage
{
  public:
    void AddMetric(uint64_t sampleKey, const TotalNetworkSample& sample);

    bool IsReportReady()
    {
        if (static_cast<int32_t>(_ringMetrics.size()) < _reportCadence)
        {
            // Configuration error, this will never succeed in reporting
            FAIL_FAST();
        }

        return (_samplesSinceLastReport >= _reportCadence);
    }

    void ReportMetrics();

    void SetBufferSize(int64_t size) { _ringMetrics.resize(size); }

    void SetReportCadence(int32_t reportCadence)
    {
        _reportCadence = reportCadence;
    }

  private:
    void IncrementWithWrap(int32_t& value, int32_t wrapSize)
    {
        value++;
        if (value >= wrapSize)
        {
            value = 0;
        }
    }

  private:
    // Timestamp & NetworkInfo pair
    std::vector<std::pair<uint64_t, TotalNetworkSample>> _ringMetrics;
    int32_t _currentRingEntry = 0;
    int32_t _oldestValidEntry = -1;
    int32_t _newestValidEntry = -1;

    int32_t _samplesSinceLastReport = 0;
    int32_t _reportCadence = 0;
};

}}}}
