// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <chrono>
#include <type_traits>

namespace Azure { namespace Device { namespace Health { namespace Plugin {

// Note: the size for network_stat_files and NetworkStats should be the same
static const char* NetworkStatsFiles[] = { "rx_bytes",   "tx_bytes",
                                           "rx_packets", "tx_packets",
                                           "rx_dropped", "tx_dropped",
                                           "multicast",  "collisions" };

enum NetworkStats
{
    BYTES_RECEIVED,
    BYTES_TRANSMITTED,
    PACKETS_RECEVED,
    PACKETS_TRANSMITTED,
    PACKETS_RECEIVED_DROPPED,
    PACKETS_TRANSMITTED_DROPPED,
    MULTICAST_PACKETS,
    PACKET_COLLISIONS,
    NUM_VALUES
};

static_assert(
    NetworkStats::NUM_VALUES == std::extent<decltype(NetworkStatsFiles)>::value,
    "Incorrect number of NetworkStats values.");

struct NetworkInterfaceInfo
{
    uint64_t Sample[static_cast<uint32_t>(NetworkStats::NUM_VALUES)] = { 0 };
    std::string InterfaceName;
};

struct TotalNetworkSample
{
    std::vector<NetworkInterfaceInfo> Interfaces;
};

}}}}
