// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <string>
#include <vector>

// CPU structures
#include <azure-device-util/FileSystem.h>
#include <azure-device-util/String.h>

#include "NetworkMetrics.h"

namespace Azure { namespace Device { namespace Health { namespace Plugin {

class NetworkStatParser
{
  public:
    NetworkStatParser() : _rootPath("/") {}

    NetworkStatParser(std::string_view rootPath) : _rootPath(rootPath) {}

    /*
        Parses every network interface on the machine and retrives the
       statistics specified by NetworkStatsFiles
     */
    bool Parse(TotalNetworkSample& sample)
    {
        std::string networkInfoPath;
        networkInfoPath.append(_rootPath);
        networkInfoPath.append("sys/class/net/");

        for (const auto& item :
             std::filesystem::directory_iterator(networkInfoPath))
        {
            NetworkInterfaceInfo networkInterfaceInfo;

            std::string folderPath = item.path().c_str();
            networkInterfaceInfo.InterfaceName =
                folderPath.substr(folderPath.find_last_of('/') + 1);

            folderPath.append("/statistics/");

            for (int i = 0; i < NetworkStats::NUM_VALUES; i++)
            {
                std::string finalPath = folderPath + NetworkStatsFiles[i];

                std::string contents;

                if (FAILED(Utils::FileSystem::ReadUTF8StringFromFile(
                        finalPath, contents)))
                {
                    continue;
                }
                networkInterfaceInfo.Sample[i] = atoll(contents.c_str());
            }
            sample.Interfaces.push_back(networkInterfaceInfo);
        }

        return true;
    }

  private:
    std::string _rootPath;
};

}}}}
