// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <system_error>

// clang-format off

enum class HealthError
{
    E_INVALID_CONFIGURATION = 1,
    E_FAILED_TO_LOAD_PLUGIN_SO = 2,
    E_FAILED_TO_LOAD_PLUGIN_EXPORTED_FUNCTION = 3,
    E_PLUGIN_FAILED_TO_CREATE_INSTANCE = 4,
    E_IGNORED_DUPLICATE_PLUGIN_NAME = 5,
};

namespace std
{
    template <>
    class is_error_code_enum<HealthError> : public true_type {};
}

class HealthErrorCat : public std::error_category
{
    const char* name() const noexcept override { return "HealthErrorCat"; }
    std::string message(int) const override { return "Some Error"; }
};

extern HealthErrorCat c_healthErrorCat;

inline std::error_code make_error_code(HealthError error)
{
    return std::error_code{ static_cast<int>(error), c_healthErrorCat };
}