// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <utility>
#include <vector>

#include <azure-device-health-plugin/plugin-dev.h>
#include <azure-device-util/Toml.h>

namespace Azure { namespace Device { namespace Health {

class PluginModule;

class PluginFactory
{
  public:
    static std::shared_ptr<PluginFactory> Create(
        std::shared_ptr<PluginModule> module,
        const adh_plugin_definition* pluginDef);

    std::string_view GetName() const;
    std::vector<std::string_view> GetConsumeTargets() const;

    const adh_plugin_definition* GetDefinition() const
    {
        return _pluginDefinition;
    }

    bool IsProducer() const;
    bool IsConsumer() const;
    bool IsLogger() const;

    adh_plugin_instance*
    CreateInstance(const std::map<std::string, std::string>& configMap);

  private:
    PluginFactory(
        std::shared_ptr<PluginModule> module,
        const adh_plugin_definition* pluginDef);

  private:
    std::shared_ptr<PluginModule> _module;
    const adh_plugin_definition* _pluginDefinition;
};

}}}
