// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <memory>
#include <string_view>

#include <azure-device-health-plugin/plugin-dev.h>

namespace Azure { namespace Device { namespace Health {

class PluginFactory;

using SamplePtr = std::shared_ptr<void>;

class PluginInstance
{
  public:
    PluginInstance(
        std::shared_ptr<PluginFactory> factory,
        adh_plugin_instance* instance);
    virtual ~PluginInstance();

    std::string_view GetName() const;

  protected:
    std::shared_ptr<PluginFactory> _factory;
    adh_plugin_instance* _instance;
};

class Counter
{
  public:
    explicit Counter(uint32_t max = 1) : _max(max) {}
    bool IncrementAndCheckForMax()
    {
        ++_counter;
        if (_counter >= _max)
        {
            _counter = 0;
            return true;
        }
        else
        {
            return false;
        }
    }

    void SetMax(uint32_t newMax) { _max = newMax; }
    void PrepareToFire() { _counter = _max - 1; }

  private:
    uint32_t _counter = 0;
    uint32_t _max;
};

class ProducerPluginInstance : public PluginInstance
{
  public:
    using PluginInstance::PluginInstance;
    SamplePtr ProduceSample(uint64_t sampleKey);

    bool ShouldProduceData();
    void SetPollQuantum(uint64_t pollQuantum)
    {
        _quantumCounter.SetMax(pollQuantum);
    }

  private:
    Counter _quantumCounter;
};

class ConsumerPluginInstance : public PluginInstance
{
  public:
    using PluginInstance::PluginInstance;
    bool
    ConsumeSample(uint64_t sampleKey, std::string_view producerName, SamplePtr sample);

    bool ShouldConsumeProducer(std::string_view producerName);
};
class LoggerPluginInstance : public PluginInstance
{
  public:
    using PluginInstance::PluginInstance;
    bool LogData(uint64_t sampleKey);

    void SetPollQuantum(uint64_t pollQuantum)
    {
        _quantumCounter.SetMax(pollQuantum);

        // Since Loggers should always fire on boot, set the counter to fire the
        // first time
        _quantumCounter.PrepareToFire();
    }

    bool ShouldLogData();

  private:
    Counter _quantumCounter;
};

}}}
