// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <dlfcn.h>
#include <memory>
#include <string>
#include <string_view>
#include <vector>

#include <azure-device-health-plugin/plugin-dev.h>
#include <azure-device-util/RAII.h>

namespace Azure { namespace Device { namespace Health {

// For a directory, loop over each found module and call Create(name)
class PluginFactory;
class PluginModule : public std::enable_shared_from_this<PluginModule>
{
  public:
    static std::shared_ptr<PluginModule> Create(std::string_view path);

    std::vector<std::shared_ptr<PluginFactory>> GetPluginFactories();

  private:
    PluginModule(std::string_view path);

    using GetPluginsFuncType = const adh_plugin_definition** (*) ();

  private:
    std::string _path;
    Azure::Device::Utils::unique_dl_module _dlModule;
    GetPluginsFuncType _getPluginsFunc;
};

}}}
