// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <array>
#include <string>

#include <azure-device-util/Permission.h>
#include <uuid/uuid.h>

namespace Azure { namespace Device { namespace Telemetry {

enum class TriggerTelemetryCategory : uint64_t
{
    None,
    Telemetry,
    Measures,
    CriticalData
};

enum TriggerTelemetryKeyword : uint64_t
{
    Telemetry = 0x0000200000000000ULL,
    Measures = 0x0000400000000000ULL,
    CriticalData = 0x0000800000000000ULL
};

enum class TriggerPersistence : int8_t
{
    Normal = 0,
    Critical
};

enum class TriggerLatency : int8_t
{
    Normal = 0,
    Realtime
};

constexpr std::array<TriggerLatency, 2> c_triggerLatencies = {
    { TriggerLatency::Normal, TriggerLatency::Realtime }
};

enum class BufferType : int32_t
{
    Normal,
    Critical,
    Realtime
};
constexpr std::array<BufferType, 3> c_bufferTypes = {
    { BufferType::Normal, BufferType::Critical, BufferType::Realtime }
};

// Invalid message ID for container/host socket
static const int64_t k_invalidAgentMessageId = -1;
static const int32_t k_invalidSocket = -1;

using TelemetryPermission = Utils::TelemetryPermission;

}}}
