// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <system_error>
#include <vector>

// Make sure to include cstddef before gsl to avoid linker errors with std vs.
// gsl byte at the moment
#include <cstddef>

#include <azure-device-util/String.h>
#include <nonstd/span.hpp>

#include "IHttpSessionManager.h"

namespace web { namespace http { namespace client {
class http_client;
}}}

namespace pplx {
class cancellation_token;
}

namespace Azure { namespace Device { namespace Telemetry {

class HttpSessionManager : public IHttpSessionManager
{
  public:
    HttpSessionManager();
    ~HttpSessionManager();

    void SetServerUrl(std::string_view serverUrl) override;

    bool SetProxy(std::string_view proxy) override;

    std::error_code DoSynchronousHttpRequest(
        std::string_view path,
        const HttpQueryMap& queryParams,
        HttpMethod httpMethod,
        const HttpHeaderMap& headers,
        const std::vector<std::byte>& payload,
        const pplx::cancellation_token& cancellationToken,
        HttpResponse& response) override;

  private:
    std::unique_ptr<web::http::client::http_client> _httpClient;
    std::string _httpProxy;
};

}}}