// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <string_view>

#include <azure-device-util/SyncEvent.h>
#include <pplx/pplx.h>

#include "IUploadScheduler.h"

namespace Azure { namespace Device { namespace Telemetry {

struct ReadResult;
class IHeartBeat;
class IAsimovHttpClient;
struct HttpResponse;
class IEventStore;
class IEventSerializer;

class IAsimovUploadWorkItem
{
  public:
    virtual ~IAsimovUploadWorkItem() {}

    virtual void Execute(pplx::cancellation_token cancelToken) = 0;

    virtual void Preempt() = 0;

    virtual UploadWorkDefinition GetWorkDefinition() const = 0;
    virtual UploadResult GetResult() const = 0;

    virtual void LogCompletionEvent() = 0;
};

class IAsimovUploadWorkItemFactory
{
  public:
    virtual ~IAsimovUploadWorkItemFactory() {}
    virtual std::shared_ptr<IAsimovUploadWorkItem> Create(
        IAsimovHttpClient& httpClient,
        IHeartBeat& heartBeat,
        IEventStore& eventStore,
        IEventSerializer& serializer,
        UploadWorkDefinition workDefinition) = 0;
};

class AsimovUploadWorkItemFactory : public IAsimovUploadWorkItemFactory
{
  public:
    std::shared_ptr<IAsimovUploadWorkItem> Create(
        IAsimovHttpClient& httpClient,
        IHeartBeat& heartBeat,
        IEventStore& eventStore,
        IEventSerializer& serializer,
        UploadWorkDefinition workDefinition) override;
};

}}}