// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <memory>
#include <mutex>
#include <string>
#include <vector>

#include <azure-device-util/IConfig.h>

#include "CommonDefs.h"

namespace Azure { namespace Device { namespace Telemetry {

class AsimovEvent;
class AsimovSharedPartAState;
class AsimovSyntheticEvent;
class IEventStore;
class IHeartBeat;
class IUploadScheduler;
class IKillSwitchManager;
class ITelemetrySettings;

class IEventSerializer
{
  public:
    struct WriteEventOptions
    {
        WriteEventOptions() {}

        bool SkipArmTimer = false;
    };

    virtual ~IEventSerializer() {}

    virtual IKillSwitchManager& GetKillSwitchManager() = 0;

    virtual bool WriteSyntheticEvent(
        std::unique_ptr<AsimovSyntheticEvent> event,
        WriteEventOptions options = {}) = 0;

    virtual bool WriteEvent(
        std::unique_ptr<AsimovEvent> event,
        WriteEventOptions options = {}) = 0;

    virtual void UpdateTelemetrySettings(
        std::unique_ptr<ITelemetrySettings> telemetrySettings) = 0;

    virtual void ReloadConfiguration(Utils::IConfig& config) = 0;

    virtual void BeginSerializing(IEventStore* eventStore) = 0;
    virtual void BeginSettingUploadTimers(IUploadScheduler* uploadQueue) = 0;
    virtual void SetHeartBeat(IHeartBeat& heartBeat) = 0;

    virtual void PollHeartbeatStatistics() {}
};

}}}