// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <memory>
#include <string>
#include <string_view>
#include <system_error>
#include <vector>

// Make sure to include cstddef before gsl to avoid linker errors with std vs.
// gsl byte at the moment
#include <cstddef>

#include <azure-device-util/String.h>
#include <nonstd/span.hpp>

namespace web { namespace http { namespace client {
class http_client;
}}}

namespace pplx {
class cancellation_token;
}

namespace Azure { namespace Device { namespace Telemetry {

enum class HttpMethod : uint8_t
{
    GET,
    POST
};

//
// If the created HttpRequest supplied an output stream, DataStream will be set.
// Otherwise, Data will be set.
//
using HttpHeaderMap =
    std::map<std::string, std::string, Utils::String::CaseInsensitiveLessThanPredicate>;

std::string FlattenHttpHeaderMap(const HttpHeaderMap& headerMap);

using HttpQueryMap =
    std::map<std::string, std::string, Utils::String::CaseInsensitiveLessThanPredicate>;

struct HttpResponse
{
    HttpHeaderMap Headers;
    std::string Data;
    uint32_t StatusCode = 0;
    std::string ETag;
    uint32_t RetryAfterSeconds = 0;
};

class IHttpSessionManager
{
  public:
    virtual ~IHttpSessionManager() {}

    virtual void SetServerUrl(std::string_view serverUrl) = 0;

    virtual bool SetProxy(std::string_view proxy) = 0;

    virtual std::error_code DoSynchronousHttpRequest(
        std::string_view path,
        const HttpQueryMap& queryParams,
        HttpMethod httpMethod,
        const HttpHeaderMap& headers,
        const std::vector<std::byte>& payload,
        const pplx::cancellation_token& cancellationToken,
        HttpResponse& response) = 0;
};

}}}
