// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <mutex>
#include <string_view>

#include "CommonDefs.h"

namespace Azure { namespace Device { namespace Telemetry {

class ITimeService;

class IKillSwitchManager
{
  public:
    virtual ~IKillSwitchManager() {}

    virtual bool
    IsKilled(std::string_view tenantToken, TriggerPersistence persistence) = 0;

    virtual void SetKillRequest(
        std::string tenantToken,
        TriggerPersistence killMode,
        std::chrono::seconds killDuration) = 0;
};

class KillSwitchManager : public IKillSwitchManager
{
  public:
    KillSwitchManager(ITimeService& timeService);

    bool
    IsKilled(std::string_view tenantToken, TriggerPersistence persistence) override;

    void SetKillRequest(
        std::string tenantToken,
        TriggerPersistence killMode,
        std::chrono::seconds killDuration) override;

  private:
    ITimeService& _timeService;

    struct KillState
    {
        TriggerPersistence Persistence = TriggerPersistence::Normal;
        std::chrono::steady_clock::time_point UntilTime;
    };

    std::recursive_mutex _mutex;
    std::map<std::string, KillState, std::less<>> _killStates;
};

}}}