// Copyright (c) Microsoft Corporation. All rights reserved.

#include "ConsumerFactory.h"

#include "IPolicyManager.h"
#include "TelemetryError.h"
#include "azure-device-util/ResultException.h"

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_JOURNALD
    #include "journald/JournalDConsumer.hpp"
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_LTTNG
    #include "lttng/LttngEventConsumer.h"
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_HWBINDER
    #include "hwbinder/HwBinderConsumer.h"
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_LOGCAT
    #include "logcat/LogcatConsumer.h"
#endif

using namespace Azure::Device::Policy;

namespace Azure { namespace Device { namespace Telemetry {

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_JOURNALD
std::unique_ptr<IEventConsumer> ConsumerFactory::CreateJournalDConsumer(
    IEventSerializer& serializer,
    IPolicyManager& policyManager,
    std::chrono::milliseconds pollInterval)
{
    return std::make_unique<JournalDConsumer>(
        serializer, policyManager, pollInterval);
}
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_LTTNG
std::unique_ptr<IEventConsumer> ConsumerFactory::CreateLttngConsumer(
    IEventSerializer& serializer,
    IPolicyManager& policyManager,
    std::chrono::milliseconds pollInterval)
{
    return std::make_unique<LttngEventConsumer>(
        serializer, policyManager, pollInterval);
}
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_HWBINDER
std::unique_ptr<IEventConsumer> ConsumerFactory::CreateHwBinderConsumer(
    IEventSerializer& serializer,
    IPolicyManager& policyManager)
{
    return std::make_unique<HwBinderConsumer>(serializer, policyManager);
}
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_LOGCAT
std::unique_ptr<IEventConsumer> ConsumerFactory::CreateLogcatConsumer(
    IEventSerializer& serializer,
    IPolicyManager& policyManager)
{
    return std::make_unique<LogcatConsumer>(serializer, policyManager);
}
#endif

}}}