// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <memory>
#include <string_view>

#include "IEventConsumer.h"
#include "IPolicyManager.h"

namespace Azure { namespace Device { namespace Telemetry {

// Defines Interface of EventStore Type
class ConsumerFactory
{
  public:
#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_JOURNALD
    static std::unique_ptr<IEventConsumer> CreateJournalDConsumer(
        IEventSerializer& serializer,
        Policy::IPolicyManager& policyManager,
        std::chrono::milliseconds pollInterval);
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_LTTNG
    static std::unique_ptr<IEventConsumer> CreateLttngConsumer(
        IEventSerializer& serializer,
        Policy::IPolicyManager& policyManager,
        std::chrono::milliseconds pollInterval);
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_HWBINDER
    static std::unique_ptr<IEventConsumer> CreateHwBinderConsumer(
        IEventSerializer& serializer,
        Policy::IPolicyManager& policyManager);
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_CONSUMER_LOGCAT
    static std::unique_ptr<IEventConsumer> CreateLogcatConsumer(
        IEventSerializer& serializer,
        Policy::IPolicyManager& policyManager);
#endif
};

}}}
