// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <atomic>
#include <chrono>
#include <optional>
#include <set>
#include <string>
#include <optional>

#include <systemd/sd-journal.h>

#include "MessageEvent.hpp"

namespace Azure { namespace Device { namespace Telemetry {

using SourceList = std::set<std::string>;

class Journal
{
  public:
    Journal(std::chrono::milliseconds pollInterval);
    ~Journal();
    void StopCollecting();
    std::optional<MessageEvent> GetNextEvent();
    void AddSource(std::string_view filter);
    void AddSourceList(const SourceList& sourceList);

  private:
    void InitJournal();
    void WaitForNextEvent();
    MessageFields LoadFields();

    sd_journal* sdj;
    std::chrono::milliseconds _pollInterval;
    std::atomic_bool _stopCollecting = false;
};

}}}