// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <atomic>
#include <chrono>
#include <functional>
#include <thread>

#include <jsonbuilder/JsonBuilder.h>

#include "AsimovEvent.h"
#include "IEventConsumer.h"
#include "IEventSerializer.h"
#include "Journal.hpp"
#include "TlgProvider.h"
#include "IPolicyManager.h"

namespace Azure { namespace Device { namespace Telemetry {

class JournalDConsumer : public IEventConsumer
{
  public:
    explicit JournalDConsumer(
        IEventSerializer& serializer,
        Policy::IPolicyManager& policyManager,
        std::chrono::milliseconds pollInterval);

    void StartConsuming() override;
    void ReloadConfiguration() override;
    void StopConsuming() override;
    void WaitForCompletion() override;

  private:
    void ConsumeTask();

  private:
    IEventSerializer& _serializer;
    Policy::IPolicyManager& _policyManager;
    Journal _journal;
    std::atomic_bool _stopCollecting = false;
    std::thread _consumerThread;
};

}}}