// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <optional>
#include <set>
#include <string>
#include <optional>

#include <jsonbuilder/JsonBuilder.h>

#include <azure-device-util/Toml.h>

#include "CommonDefs.h"
#include "IPolicyManager.h"
#include "StaticParser.hpp"
#include "ConsumerConfig.hpp"

using SourceList = std::set<std::string>;

namespace Azure { namespace Device { namespace Telemetry {

class JournalDConsumerConfig
{
  public:
    // The journald field _COMM is limited to 16 characters by the Kernel.
    // This includes the null terminator, so there are only 15 usable
    // characters. All declared _COMM filters will be truncated to this size,
    // otherwise they will never match.
    static constexpr size_t c_CommMaxSize = 15;

    JournalDConsumerConfig(Policy::IPolicyManager& policyManager);

    // Regular members
    void AddConfigFile(const toml::value& data);
    bool HasCollectionConfigured();
    std::optional<TelemetryPermission>
    GetTelemetryPermissionFromJsonBuilder(jsonbuilder::JsonBuilder& jsonBuilder);
    const SourceList& GetJournalSourceList() const;
    void Reset();
    const StaticParserRuleSet& GetStaticParserRuleSet() const;

  private:
    bool ReadJournalFilter(
        const toml::value& data,
        std::string fieldName,
        size_t maxChars = std::string::npos);

  private:
    SourceList _journalSources;
    ConsumerConfig _consumerConfig;
};

class JournalDConfigurationException : public std::runtime_error
{
    using std::runtime_error::runtime_error;
};

}}}
