// Copyright (c) Microsoft Corporation. All rights reserved.

#include "Router.hpp"
#include <optional>

namespace Azure { namespace Device { namespace Telemetry {

Router::Router(IEventSerializer& serializer, JournalDConsumerConfig& consumerConfig)
    : _serializer(serializer)
    , _consumerConfig(consumerConfig)
    , _staticParser(consumerConfig.GetStaticParserRuleSet())
{}

std::optional<jsonbuilder::JsonBuilder>
Router::GetEventJsonBuilder(const MessageEvent& event)
{
    if (event.GetTagLocation() != MessageEvent::TagLocation::None)
    {
        // Check for dynamic definitions
        return _dynParser.ParseEvent(event);
    }
    else
    {
        // Check for static definitions
        return _staticParser.ParseEvent(event);
    }
}

void Router::RouteMessageEvent(const MessageEvent& event)
{
    auto optJsonBuilder = GetEventJsonBuilder(event);
    if (!optJsonBuilder)
    {
        return;
    }

    auto optUploadVolume =
        _consumerConfig.GetTelemetryPermissionFromJsonBuilder(*optJsonBuilder);
    if (!optUploadVolume)
    {
        return;
    }

    std::unique_ptr<AsimovEvent> asimovEvent =
        AsimovEvent::CreateFromLttngConsumer(
            std::move(*optJsonBuilder), *optUploadVolume);

    _serializer.WriteEvent(std::move(asimovEvent));
}

}}}
