// Copyright (c) Microsoft Corporation. All rights reserved.

#include "LttngApi.h"

#include <lttng/lttng-error.h>
#include <lttng/lttng.h>
#include <lttng/session.h>

namespace Azure { namespace Device { namespace Telemetry {

int LttngApi::CreateSessionLive(
    const char* name,
    const char* url,
    unsigned int timer_interval)
{
    return lttng_create_session_live(name, url, timer_interval);
};

int LttngApi::DestroySession(const char* name)
{
    return lttng_destroy_session(name);
}

lttng_handle*
LttngApi::CreateHandle(const char* session_name, lttng_domain* domain)
{
    return lttng_create_handle(session_name, domain);
}

void LttngApi::DestroyHandle(lttng_handle* handle)
{
    lttng_destroy_handle(handle);
}

int LttngApi::AddContext(
    lttng_handle* handle,
    lttng_event_context* ctx,
    const char* event_name,
    const char* channel_name)
{
    return lttng_add_context(handle, ctx, event_name, channel_name);
}

lttng_channel* LttngApi::CreateChannel()
{
    lttng_domain domain = lttng_domain();
    domain.type = LTTNG_DOMAIN_UST;
    domain.buf_type = LTTNG_BUFFER_PER_UID;

    return lttng_channel_create(&domain);
}

void LttngApi::DestroyChannel(lttng_channel* channel)
{
    lttng_channel_destroy(channel);
}

int LttngApi::EnableChannel(lttng_handle* handle, lttng_channel* channel)
{
    return lttng_enable_channel(handle, channel);
}
void LttngApi::DisableChannel(lttng_channel* channel)
{
    lttng_channel_destroy(channel);
}

int LttngApi::StartTracing(const char* session_name)
{
    return lttng_start_tracing(session_name);
}

int LttngApi::StopTracing(const char* session_name)
{
    return lttng_stop_tracing(session_name);
}

int LttngApi::EnableEvent(lttng_handle* handle, lttng_event* ev, const char* channel_name)
{
    return lttng_enable_event(handle, ev, channel_name);
}

int LttngApi::DisableEvent(
    lttng_handle* handle,
    const char* event_name,
    const char* channel_name)
{
    return lttng_disable_event(handle, event_name, channel_name);
}

}}}
