// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

struct lttng_domain;
struct lttng_event;
struct lttng_event_context;
struct lttng_handle;
struct lttng_channel;

namespace Azure { namespace Device { namespace Telemetry {

class ILttngApi
{
  public:
    virtual ~ILttngApi() {}

    virtual int CreateSessionLive(
        const char* name,
        const char* url,
        unsigned int timer_interval) = 0;

    virtual int DestroySession(const char* name) = 0;

    virtual lttng_handle*
    CreateHandle(const char* session_name, lttng_domain* domain) = 0;

    virtual void DestroyHandle(lttng_handle* handle) = 0;

    virtual int AddContext(
        lttng_handle* handle,
        lttng_event_context* ctx,
        const char* event_name,
        const char* channel_name) = 0;

    virtual lttng_channel* CreateChannel() = 0;
    virtual void DestroyChannel(lttng_channel* channel) = 0;

    virtual int EnableChannel(lttng_handle* handle, lttng_channel* channel) = 0;
    virtual void DisableChannel(lttng_channel* channel) = 0;

    virtual int StartTracing(const char* session_name) = 0;
    virtual int StopTracing(const char* session_name) = 0;

    virtual int
    EnableEvent(lttng_handle* handle, lttng_event* ev, const char* channel_name) = 0;

    virtual int DisableEvent(
        lttng_handle* handle,
        const char* event_name,
        const char* channel_name) = 0;
};

class LttngApi : public ILttngApi
{
  public:
    int CreateSessionLive(
        const char* name,
        const char* url,
        unsigned int timer_interval) override;

    int DestroySession(const char* name) override;

    lttng_handle*
    CreateHandle(const char* session_name, lttng_domain* domain) override;

    void DestroyHandle(lttng_handle* handle) override;

    int AddContext(
        lttng_handle* handle,
        lttng_event_context* ctx,
        const char* event_name,
        const char* channel_name) override;

    lttng_channel* CreateChannel() override;
    void DestroyChannel(lttng_channel* channel) override;

    int EnableChannel(lttng_handle* handle, lttng_channel* channel) override;
    void DisableChannel(lttng_channel* channel) override;

    int StartTracing(const char* session_name) override;
    int StopTracing(const char* session_name) override;

    int
    EnableEvent(lttng_handle* handle, lttng_event* ev, const char* channel_name)
        override;

    int DisableEvent(
        lttng_handle* handle,
        const char* event_name,
        const char* channel_name) override;
};

}}}
