// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <atomic>
#include <chrono>
#include <thread>

#include "AsimovEvent.h"
#include "IEventConsumer.h"
#include "IEventSerializer.h"
#include "IPolicyManager.h"

namespace LttngConsume {
class LttngConsumer;
}

namespace Azure { namespace Device { namespace Telemetry {

class LttngSession;

class LttngEventConsumer : public IEventConsumer
{
  public:
    explicit LttngEventConsumer(
        IEventSerializer& serializer,
        Policy::IPolicyManager& policyManager,
        std::chrono::milliseconds pollInterval);

    ~LttngEventConsumer();

    void StartConsuming() override;
    void ReloadConfiguration() override;
    void StopConsuming() override;
    void WaitForCompletion() override;

    static std::string
    RemoveEventSourceEnvelope(jsonbuilder::JsonBuilder& builder);

  private:
    void ConsumeTask();

    static void OnNewJsonBuilder(
        IEventSerializer& _serializer,
        LttngSession& _session,
        jsonbuilder::JsonBuilder&& builder);

    static std::string_view
    GetCleanedUpLttngName(const jsonbuilder::JsonBuilder& builder);

    static void AddParsedPayloadToData(
        jsonbuilder::JsonBuilder& builder,
        jsonbuilder::JsonBuilder::iterator dataItr,
        std::string_view payloadStr);

    static std::string UnescapeJson(std::string_view escapedJson);

  private:
    IEventSerializer& _serializer;
    Policy::IPolicyManager& _policyManager;
    std::chrono::milliseconds _pollInterval;
    std::unique_ptr<LttngSession> _session;
    std::unique_ptr<LttngConsume::LttngConsumer> _consumer;
    std::thread _consumerThread;
};

}}}