// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <chrono>
#include <map>
#include <memory>
#include <set>
#include <string>
#include <string_view>

#include "CommonDefs.h"

struct lttng_handle;

namespace Azure { namespace Device { namespace Telemetry {

class ILttngApi;

using ListeningRuleSet =
    std::map<TelemetryPermission, std::set<std::string, std::less<>>>;

class LttngSession
{
  public:
    LttngSession(
        std::unique_ptr<ILttngApi> lttngApi,
        std::string_view sessionName,
        std::string_view hostName,
        std::chrono::milliseconds retryInterval,
        uint32_t retriesAllowed);
    ~LttngSession();

    void EnableDotNetEventSourceEvents();
    void AddRuleSet(std::string_view fileName, ListeningRuleSet&& ruleSet);

    void SetTelemetryPermission(TelemetryPermission telemetryPermission);

    void Start();
    void Stop();

    std::optional<TelemetryPermission>
    GetMinimumRequiredTelemetryPermissionForEvent(std::string_view eventName);

  public:
    static std::string
    MakeConnectionString(std::string_view sessionName, std::string_view hostName);

  public:
    static constexpr std::string_view c_channelName = "userchannel";
    static constexpr std::string_view c_eventSourceLttngName =
        "DotNETRuntime:EventSource";

  private:
    void EnableEvents(const std::set<std::string, std::less<>>& eventsToEnable);
    void DisableEvents(const std::set<std::string, std::less<>>& eventsToDisable);

  private:
    std::unique_ptr<ILttngApi> _lttngApi;
    TelemetryPermission _telemetryPermission = TelemetryPermission::Required;
    std::map<std::string, ListeningRuleSet, std::less<>> _fileNameToRuleSets;
    std::string _sessionName;
    lttng_handle* _lttngHandle;
};

}}}
