// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <optional>
#include <set>
#include <string>

#include <jsonbuilder/JsonBuilder.h>

#include <azure-device-util/Toml.h>

#include "CommonDefs.h"
#include "IPolicyManager.h"
#include "StaticParser.hpp"

namespace Azure { namespace Device { namespace Telemetry {

class ConsumerConfig
{
  public:
    enum class EventType
    {
        Tagged,
        Static
    };

    ConsumerConfig(Policy::IPolicyManager& policyManager);

    // Regular members
    void AddConfigFile(const toml::value& data);
    bool HasCollectionConfigured();
    const StaticParserRuleSet& GetStaticParserRuleSet() const;
    void Reset();
    std::optional<TelemetryPermission>
    GetTelemetryPermissionFromJsonBuilder(
        jsonbuilder::JsonBuilder& jsonBuilder);

  private:
    bool
    RegisterEventFromToml(const toml::value& eventDefinition, EventType type);

  private:
    std::map<std::string, TelemetryPermission, std::less<>> _eventList;
    Policy::IPolicyManager& _policyManager;
    StaticParserRuleSet _staticRules;
};

}}}
