// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <regex>
#include <string>
#include <optional>

#include <jsonbuilder/JsonBuilder.h>

#include "MessageEvent.hpp"

namespace Azure { namespace Device { namespace Telemetry {

class DynamicParser
{
  public:
    static constexpr auto c_EventRegexStart =
        R"regex(^@ADHS@([A-Z][0-9a-zA-Z]*([.|-][A-Z][0-9a-zA-Z]*)*):([A-Z][0-9a-zA-Z]*)(#([a-zA-Z0-9_]+))?@ADHS@((.|\n)*)$)regex";

    // Groups corresponding to c_EventRegexStart
    enum class RegexStartGroups
    {
        Provider = 1,
        Name = 3,
        Mode = 5,
        Content = 6,
    };

    static constexpr auto c_EventRegexEnd =
        R"regex(^((.|\n)*)@ADHS@([A-Z][0-9a-zA-Z]*([.|-][A-Z][0-9a-zA-Z]*)*):([A-Z][0-9a-zA-Z]*)(#([a-zA-Z0-9_]+))?@ADHS@$)regex";

    // Groups corresponding to c_EventRegexEnd
    enum class RegexEndGroups
    {
        Provider = 3,
        Name = 5,
        Mode = 7,
        Content = 1,
    };

    static constexpr auto c_KeyValueRegex =
        R"regex(([a-zA-Z0-9_]+)=([a-zA-Z0-9_]+))regex";

    DynamicParser();

    std::optional<jsonbuilder::JsonBuilder> ParseEvent(const MessageEvent& event);

  private:
    std::optional<jsonbuilder::JsonBuilder> BuildEventJsonBuilder(
        const std::string& providerName,
        const std::string& eventName,
        std::string_view mode,
        std::string_view content);

    bool ParseData(
        std::string_view mode,
        std::string_view content,
        jsonbuilder::JsonBuilder& jb,
        jsonbuilder::JsonIterator itr);

    bool ParseKeyValue(
        std::string_view content,
        jsonbuilder::JsonBuilder& jb,
        jsonbuilder::JsonIterator itr);

    bool ParseJson(
        std::string_view content,
        jsonbuilder::JsonBuilder& jb,
        jsonbuilder::JsonIterator itr);

    std::regex _regexTagStart;
    std::regex _regexTagEnd;
    std::regex _regexKeyValue;
};

}}}
