// Copyright (c) Microsoft Corporation. All rights reserved.

#include "MessageEvent.hpp"
#include <optional>

#include <azure-device-util/String.h>

namespace Azure { namespace Device { namespace Telemetry {

MessageEvent::MessageEvent(
    MessageFields fields,
    std::chrono::system_clock::time_point partATime)
    : _fields(std::move(fields)), _timestamp(std::move(partATime))
{
    FindTag();
};

void MessageEvent::FindTag()
{
    if (Utils::String::IStartsWith(Message(), c_EventTag))
    {
        _tagLocation = TagLocation::Start;
    }
    else if (Utils::String::IEndsWith(Message(), c_EventTag))
    {
        _tagLocation = TagLocation::End;
    }
    else
    {
        _tagLocation = TagLocation::None;
    }
}

std::string_view MessageEvent::Message() const
{
    auto optMessage = Field("MESSAGE");
    return optMessage ? *optMessage : "";
}

std::optional<std::string_view>
MessageEvent::Field(std::string_view fieldName) const
{
    if (auto itr = _fields.find(fieldName); itr != _fields.end())
    {
        return itr->second;
    }

    return {};
}

std::chrono::system_clock::time_point MessageEvent::GetPartATime() const
{
    return _timestamp;
}

}}}