// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>
#include <string>
#include <algorithm>
#include <chrono>
#include <optional>

namespace Azure { namespace Device { namespace Telemetry {

using MessageFields = std::map<std::string, std::string, std::less<>>;

class MessageEvent
{
  public:
    static constexpr std::string_view c_EventTag = "@ADHS@";
    enum class TagLocation
    {
        None,
        Start,
        End
    };

    MessageEvent(MessageFields messageFields, std::chrono::system_clock::time_point partATime);

    std::string_view Message() const;
    std::optional<std::string_view> Field(std::string_view fieldName) const;
    TagLocation GetTagLocation() const { return _tagLocation; }
    std::chrono::system_clock::time_point GetPartATime() const;

  private:
    void FindTag();

    MessageFields _fields;
    TagLocation _tagLocation = TagLocation::None;
    std::chrono::system_clock::time_point _timestamp;
};

}}}