// Copyright (c) Microsoft Corporation. All rights reserved.

#include "StaticParser.hpp"
#include <optional>

#include <azure-device-util/FileSystem.h>
#include <azure-device-util/ResultException.h>

namespace Azure { namespace Device { namespace Telemetry {

StaticParser::StaticParser(const StaticParserRuleSet& staticRules)
    : _rules(staticRules)
{}

std::optional<jsonbuilder::JsonBuilder>
StaticParser::ParseEvent(const MessageEvent& event)
{
    for (auto& rule : _rules)
    {
        // Check if event matches journal field filter
        if (!rule.CheckFilter(event))
        {
            continue;
        }

        // Match regex
        auto result = rule.MatchEvent(event);
        if (result)
        {
            return *result;
        }
    }

    return {};
}

}}}
