// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <regex>
#include <string>
#include <optional>

#include <jsonbuilder/JsonBuilder.h>

#include <azure-device-util/Toml.h>

#include "MessageEvent.hpp"
#include "CommonDefs.h"

namespace Azure { namespace Device { namespace Telemetry {

using CollectorRuleFieldPairs = std::vector<std::pair<std::string, std::string>>;
using CollectorRuleGroupPairs = std::vector<std::pair<uint32_t, std::string>>;

class StaticParserRule
{
  public:
    static StaticParserRule FromTomlTable(const toml::value& table);

    StaticParserRule(
        std::string providerName,
        std::string eventName,
        std::optional<uint32_t> isCoreData,
        std::optional<uint32_t> privacyTags,
        std::optional<uint64_t> keyword,
        std::string regexStr);

    bool CheckFilter(const MessageEvent& event) const;

    std::optional<jsonbuilder::JsonBuilder>
    MatchEvent(const MessageEvent& event) const;

  private:
    std::string _providerName;
    std::string _eventName;
    std::optional<uint32_t> _isCoreData;
    std::optional<uint32_t> _privacyTags;
    std::optional<uint64_t> _keyword;
    std::regex _regex;
    CollectorRuleGroupPairs _groups;
    CollectorRuleFieldPairs _fields;
};

}}}
