// Copyright (c) Microsoft Corporation. All rights reserved.

#include "AppState.h"

#include <unistd.h>

#include <azure-device-util/FileSystem.h>
#include <azure-device-util/ResultException.h>

using namespace Azure::Device::Utils;

namespace Azure { namespace Device { namespace Telemetry {

/*++
    Gets the the runtime data directory for telemetry data.

    If the user is root, this will be '/var/lib/azure-device-telemetryd'.
    To support running as a regular user, use $HOME/.azure-device-telemetryd.
--*/
std::filesystem::path AppState::GetRuntimeDataDirectory()
{
    // Use a hard-coded path for root user
    if (getuid() == 0)
    {
        std::filesystem::path runtimeDataPath =
            FileSystem::GetLocalStateDirectory();
        runtimeDataPath /= "lib/azure-device-telemetryd";
        return runtimeDataPath;
    }

    const char* homeDir = getenv("HOME");
    if (homeDir != nullptr)
    {
        std::filesystem::path runtimeDataPath{ homeDir };
        runtimeDataPath /= ".azure-device-telemetryd";
        return runtimeDataPath;
    }

    FAIL_FAST();
}

}}}
