// Copyright (c) Microsoft Corporation. All rights reserved.

#include "AsimovSyntheticEvent.h"

using namespace jsonbuilder;

namespace Azure { namespace Device { namespace Telemetry {

AsimovSyntheticEvent::AsimovSyntheticEvent(
    std::string_view eventName,
    TriggerPersistence persistence,
    TriggerLatency latency,
    TelemetryPermission minTelemetryPermission,
    PrivacyTags privacyTags,
    TriggerTelemetryCategory category)
{
    _persistence = persistence;
    _latency = latency;
    _minTelemetryPermission = minTelemetryPermission;
    _telemetryCategory = category;

    std::string name;
    name = "AzureDeviceTelemetry.";

    name.append(eventName.data(), eventName.size());

    auto timestamp = std::chrono::system_clock::now();

    auto partAItr = _jsonBuilder.push_back(_jsonBuilder.root(), "a", JsonObject);
    _jsonBuilder.push_back(partAItr, "ver", "4.0");
    _jsonBuilder.push_back(partAItr, "name", name);
    _jsonBuilder.push_back(partAItr, "time", timestamp);

    auto extItr = _jsonBuilder.push_back(partAItr, "ext", JsonObject);
    _jsonBuilder.push_back(extItr, "utc", JsonObject);
    auto metadataItr = _jsonBuilder.push_back(extItr, "metadata", JsonObject);

    _partCItr = _jsonBuilder.push_back(_jsonBuilder.root(), "data", JsonObject);

    _jsonBuilder.push_back(
        metadataItr, "privTags", static_cast<uint64_t>(privacyTags));

    // Initialize the underlying keywords for the category
    if (_telemetryCategory == TriggerTelemetryCategory::Telemetry)
    {
        _keywords = TriggerTelemetryKeyword::Telemetry;
    }
    else if (_telemetryCategory == TriggerTelemetryCategory::Measures)
    {
        _keywords = TriggerTelemetryKeyword::Measures;
    }
    else if (_telemetryCategory == TriggerTelemetryCategory::CriticalData)
    {
        _keywords = TriggerTelemetryKeyword::CriticalData;
    }

    // Add an app ID for synthetics
    jsonbuilder::JsonConstIterator appItr =
        _jsonBuilder.push_back(extItr, "app", JsonObject);

    _jsonBuilder.push_back(appItr, "id", "L:azure-device-telemetryd");
}

}}}
