// Copyright (c) Microsoft Corporation. All rights reserved.

#include "DirtyShutdownMonitor.h"

#include "azure-device-util/AndroidLogHelper.h"

namespace Azure { namespace Device { namespace Telemetry {

DirtyShutdownMonitor::DirtyShutdownMonitor()
{
    _runningFlagFile = AppState::GetRuntimeDataDirectory() / "telemetryd.lock";
}

bool DirtyShutdownMonitor::RunningFlagExists()
{
    return Utils::FileSystem::FileExists(_runningFlagFile.string());
}

void DirtyShutdownMonitor::SetRunningFlag()
{
    // This is best effort. If we fail here, we simply won't track dirty
    // shutdown for this boot
    ADHS_LOG("Writing running flag");
    Utils::FileSystem::WriteUTF8StringToFile(_runningFlagFile.string(), "");
}

void DirtyShutdownMonitor::ClearRunningFlag()
{
    ADHS_LOG("Clear running flag");
    Utils::FileSystem::DeleteAllFilesWithPattern(
        AppState::GetRuntimeDataDirectory().string(), "telemetryd.lock");
}

}}}