// Copyright (c) Microsoft Corporation. All rights reserved.

#include "SerializerFactory.h"

#include "TelemetryError.h"
#include "azure-device-util/ResultException.h"

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_ASIMOV
    #include "asimov/AsimovEventSerializer.h"
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_BONDI
    #include "bondi/BondiEventSerializer.h"
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_HVSOCKET
    #include "hvsocket/HvSocketSerializer.h"
#endif

using namespace Azure::Device::Policy;

namespace Azure { namespace Device { namespace Telemetry {

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_ASIMOV
std::unique_ptr<IEventSerializer> SerializerFactory::CreateAsimovSerializer(
    IPolicyManager& policyManager,
    IKillSwitchManager& killSwitchManager,
    Utils::IConfig& config)
{
    return std::make_unique<AsimovEventSerializer>(
        policyManager, killSwitchManager, config);
}
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_BONDI
std::unique_ptr<IEventSerializer> SerializerFactory::CreateBondiSerializer(
    IPolicyManager& policyManager,
    IKillSwitchManager& killSwitchManager,
    Utils::IConfig& config)
{
    return std::make_unique<BondiEventSerializer>(
        policyManager, killSwitchManager, config);
}
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_HVSOCKET
std::unique_ptr<IEventSerializer> SerializerFactory::CreateHvSocketSerializer(
    RemotePolicyManager& policyManager,
    IKillSwitchManager& killSwitchManager,
    Utils::IConfig& config)
{
    return std::make_unique<HvSocketSerializer>(
        policyManager, killSwitchManager, config);
}
#endif

}}}