// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <memory>
#include <string_view>

#include "IEventSerializer.h"

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_ASIMOV
    #include "IPolicyManager.h"
#endif

// TODO: Can we OR this with the ASIMOV macro?
#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_BONDI
    #include "IPolicyManager.h"
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_HVSOCKET
    #include "RemotePolicyManager.h"
#endif

namespace Azure { namespace Device { namespace Telemetry {

// Defines Interface of EventStore Type
class SerializerFactory
{
  public:
#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_ASIMOV
    static std::unique_ptr<IEventSerializer> CreateAsimovSerializer(
        Policy::IPolicyManager& policyManager,
        IKillSwitchManager& killSwitchManager,
        Utils::IConfig& config);
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_BONDI
    static std::unique_ptr<IEventSerializer> CreateBondiSerializer(
        Policy::IPolicyManager& policyManager,
        IKillSwitchManager& killSwitchManager,
        Utils::IConfig& config);
#endif

#ifdef AZURE_DEVICE_TELEMETRY_USE_SERIALIZER_HVSOCKET
    static std::unique_ptr<IEventSerializer> CreateHvSocketSerializer(
        Policy::RemotePolicyManager& policyManager,
        IKillSwitchManager& killSwitchManager,
        Utils::IConfig& config);
#endif
};

}}}
