// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <optional>
#include <string_view>

#include "SettingsDataTypes.h"

namespace Azure { namespace Device { namespace Telemetry {

class ISettingsRepository
{
  public:
    virtual ~ISettingsRepository() = default;

    virtual bool
    ContentExists(const SettingsNamespaceConfig& settingsNamespace) = 0;
    virtual bool
    MetadataExists(const SettingsNamespaceConfig& settingsNamespace) = 0;

    virtual std::optional<SettingsNamespaceContent>
    ReadContent(const SettingsNamespaceConfig& settingsNamespace) = 0;

    virtual std::optional<SettingsNamespaceMetadata>
    ReadMetadata(const SettingsNamespaceConfig& settingsNamespace) = 0;

    virtual void WriteContent(
        const SettingsNamespaceConfig& settingsNamespace,
        std::string_view content) = 0;
    virtual void WriteMetadata(
        const SettingsNamespaceConfig& settingsNamespace,
        const SettingsNamespaceMetadata& metadata) = 0;
};

}}}
