// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include "AsimovSharedPartAState.h"
#include "HttpSessionManager.h"

namespace pplx {
class cancellation_token;
}

namespace Azure { namespace Device { namespace Telemetry {

class IOneSettingHttpClient
{
  public:
    virtual ~IOneSettingHttpClient() = default;

    virtual std::error_code DownloadNamespace(
        std::string_view partner,
        std::string_view feature,
        std::string_view etag,
        const HttpQueryMap& queryParams,
        const pplx::cancellation_token& cancellationToken,
        HttpResponse& response) = 0;
};

class OneSettingsHttpClient : public IOneSettingHttpClient
{
  public:
    explicit OneSettingsHttpClient(IHttpSessionManager& httpSessionManager);

    std::error_code DownloadNamespace(
        std::string_view partner,
        std::string_view feature,
        std::string_view etag,
        const HttpQueryMap& queryParams,
        const pplx::cancellation_token& cancellationToken,
        HttpResponse& response) override;

    static HttpQueryMap CreateOneSettingsQueryMap(
        const AsimovSharedPartAState::OsReleaseData& osReleaseData);

  private:
    static constexpr std::string_view c_oneSettingsProdServer =
        "settings.data.microsoft.com";
    static constexpr std::string_view c_oneSettingsPreProdServer =
        "settings-ppe.data.microsoft.com";

  private:
    IHttpSessionManager& _httpSessionManager;
    HttpHeaderMap _requestHeaders;
};
}}}