// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <chrono>
#include <map>
#include <string>

#include <azure-device-util/String.h>

namespace Azure { namespace Device { namespace Telemetry {

enum class SettingsNamespaceType
{
    Telemetry
};

std::string SettingsNamespaceTypeToString(SettingsNamespaceType type);

struct SettingsNamespaceConfig
{
    std::string Feature;
    std::string Partner;
    SettingsNamespaceType Type = SettingsNamespaceType::Telemetry;

    std::string ToString() const;
};

bool operator<(
    const SettingsNamespaceConfig& left,
    const SettingsNamespaceConfig& right);

struct SettingsNamespaceMetadata
{
    std::string ETag;
    std::chrono::system_clock::time_point LastDownloadTime;
};

using SettingsNamespaceContent =
    std::map<std::string, std::string, Utils::String::CaseInsensitiveLessThanPredicate>;

struct SettingsNamespaceState
{
    SettingsNamespaceConfig Config;
    SettingsNamespaceMetadata Metadata;
    SettingsNamespaceContent Content;
};

}}}