// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <optional>
#include <string>
#include <string_view>
#include <system_error>
#include <vector>

#include <pplx/pplx.h>

#include "IHttpSessionManager.h"
#include "SettingsDataTypes.h"

namespace Azure { namespace Device { namespace Telemetry {

class IHeartBeat;
class IEventSerializer;
class ISettingsRepository;
struct HttpResponse;

class SettingsDownloader
{
  public:
    SettingsDownloader(
        IHeartBeat& heartBeat,
        IEventSerializer& serializer,
        IHttpSessionManager& httpSessionManager,
        ISettingsRepository& repository,
        HttpQueryMap queryMap);

    void DownloadAllNamespacesToRepository(
        const std::vector<SettingsNamespaceState>& oldNamespaceStates,
        pplx::cancellation_token cancelToken);

  private:
    struct SettingDownloadResult
    {
        std::string Content;
        std::string ETag;
    };
    std::optional<SettingDownloadResult> DownloadNamespace(
        const SettingsNamespaceConfig& settingNamespace,
        std::string_view etag,
        pplx::cancellation_token cancelToken);
    void HandleNetworkError(std::error_code result);

    std::optional<SettingDownloadResult> HandleNetworkSuccess(
        const SettingsNamespaceConfig& settingNamespace,
        const HttpResponse& response);

  private:
    IHeartBeat& _heartBeat;
    IEventSerializer& _serializer;
    IHttpSessionManager& _httpSessionManager;
    ISettingsRepository& _settingsRepository;
    HttpQueryMap _queryParamMap;
};

}}}
