// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <memory>
#include <vector>

#include "SettingsDataTypes.h"

namespace Azure { namespace Device { namespace Telemetry {

class IEventSerializer;
class ITelemetrySettings;
class ISettingsRepository;

class SettingsLoader
{
  public:
    SettingsLoader(
        ISettingsRepository& settingsRepository,
        std::vector<SettingsNamespaceConfig> namespaceConfigs);

    void LoadAllNamespacesFromRepository();
    std::unique_ptr<ITelemetrySettings> BuildTelemetrySettingsFromCurrentState();

    const std::vector<SettingsNamespaceState>& GetStates() const
    {
        return _loadedStates;
    }

  private:
    ISettingsRepository& _settingsRepository;
    std::vector<SettingsNamespaceConfig> _configs;

    std::vector<SettingsNamespaceState> _loadedStates;
};

}}}
