// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <optional>
#include <string>
#include <string_view>
#include <system_error>
#include <thread>
#include <vector>

#include <boost/asio/io_service.hpp>
#include <boost/asio/steady_timer.hpp>
#include <pplx/pplx.h>

#include "HttpSessionManager.h"
#include "SettingsConfigReader.h"
#include "SettingsDownloader.h"
#include "SettingsLoader.h"
#include "SettingsRepository.h"

namespace Azure { namespace Device { namespace Utils {
class IConfig;
class ConfigPath;
}}}

namespace Azure { namespace Device { namespace Telemetry {

class IHeartBeat;
class IEventSerializer;

class SettingsManager
{
  public:
    SettingsManager(
        IHeartBeat& heartBeat,
        IEventSerializer& serializer,
        Utils::IConfig& config);

    ~SettingsManager();

    void StartTimers();
    void EndTimers();

  private:
    void OnTimerExpiry();
    void ArmTimerFromNow(std::chrono::steady_clock::duration duration);
    void LoadSettingsAndPublish();

    HttpSessionManager _httpSessionManager;
    SettingsRepository _settingsRepository;
    IEventSerializer& _eventSerializer;

    std::unique_ptr<SettingsLoader> _loader;
    std::unique_ptr<SettingsDownloader> _downloader;

    pplx::cancellation_token_source _shutdownTokenSource;
    std::thread _downloadThread;
    boost::asio::io_service _downloadTimerIoService;
    boost::asio::steady_timer _downloadTimer{ _downloadTimerIoService };

    constexpr static std::chrono::hours c_defaultDownloadFrequency{ 24 };
};

}}}
