// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <string_view>

#include "ISettingsRepository.h"
#include "SettingsDataTypes.h"

namespace Azure { namespace Device { namespace Telemetry {

class SettingsRepository : public ISettingsRepository
{
  public:
    SettingsRepository(std::filesystem::path settingsDirectory);

    bool ContentExists(const SettingsNamespaceConfig& settingsNamespace) override;
    bool
    MetadataExists(const SettingsNamespaceConfig& settingsNamespace) override;

    std::optional<SettingsNamespaceContent>
    ReadContent(const SettingsNamespaceConfig& settingsNamespace) override;

    std::optional<SettingsNamespaceMetadata>
    ReadMetadata(const SettingsNamespaceConfig& settingsNamespace) override;

    void WriteContent(
        const SettingsNamespaceConfig& settingsNamespace,
        std::string_view content) override;
    void WriteMetadata(
        const SettingsNamespaceConfig& settingsNamespace,
        const SettingsNamespaceMetadata& metadata) override;

  private:
    enum class FileType
    {
        Temp,
        Final
    };

    bool ExistsHelper(
        const SettingsNamespaceConfig& settingsNamespace,
        std::string_view dataType);

    void WriteHelper(
        std::string_view partner,
        std::string_view feature,
        std::string_view dataType,
        std::string_view fileData);

    std::filesystem::path CreateFullFileSystemPath(
        std::string_view partner,
        std::string_view feature,
        std::string_view dataType,
        FileType fileType);

    constexpr static std::string_view c_content = "content";
    constexpr static std::string_view c_metadata = "metadata";

  private:
    std::filesystem::path _settingsDirectory;
};

}}}
