// Copyright (c) Microsoft Corporation. All rights reserved.

#include "SettingsSerializers.h"
#include <optional>

#include <nlohmann/json.hpp>

#include "SettingsRepository.h"

namespace Azure { namespace Device { namespace Telemetry {

std::string SettingsNamespaceContentSerializer::ToJsonString(
    const SettingsNamespaceContent& content)
{
    nlohmann::json j;
    j["settings"] = content;
    return j.dump();
}

std::optional<SettingsNamespaceContent>
SettingsNamespaceContentSerializer::FromJsonString(std::string_view contentJson)
{
    try
    {
        nlohmann::json jsonObj = nlohmann::json::parse(contentJson);

        auto itr = jsonObj.find("settings");
        if (itr != jsonObj.end())
        {
            return itr->get<SettingsNamespaceContent>();
        }
        else
        {
            // It is valid for an empty downloaded namespace from
            // OneSettings to have no settings subobject at all
            return SettingsNamespaceContent{};
        }
    }
    CATCH_LOG()
    return {};
}

std::string SettingsNamespaceMetadataSerializer::ToJsonString(
    const SettingsNamespaceMetadata& metadata)
{
    nlohmann::json j;
    j["ETag"] = metadata.ETag;
    j["LastDownloadTime"] = metadata.LastDownloadTime.time_since_epoch().count();
    return j.dump();
}

std::optional<SettingsNamespaceMetadata>
SettingsNamespaceMetadataSerializer::FromJsonString(std::string_view metadataJson)
{
    try
    {
        SettingsNamespaceMetadata metadata;

        nlohmann::json jsonObj = nlohmann::json::parse(metadataJson);

        metadata.ETag = jsonObj.at("ETag").get<std::string>();

        int64_t systemClockTicks = jsonObj.at("LastDownloadTime").get<int64_t>();
        std::chrono::system_clock::duration dur{ systemClockTicks };
        metadata.LastDownloadTime = std::chrono::system_clock::time_point{ dur };

        return metadata;
    }
    CATCH_LOG()
    return {};
}

}}}
