// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <map>

#include <azure-device-util/String.h>

#include "ITelemetrySettings.h"
#include "SettingsDataTypes.h"

namespace Azure { namespace Device { namespace Telemetry {

class TelemetrySettings : public ITelemetrySettings
{
  public:
    bool AddSetting(std::string_view settingName, std::string_view settingValue);

    TelemetrySettingOverrides GetTelemetryEventSettings(
        Azure::Device::Utils::TelemetryPermission permissionLevel,
        std::string_view providerName,
        std::string_view eventName) override;

  private:
    std::optional<TelemetrySettingOverrides> BuildCommandOverrides(
        std::string_view settingCommandName,
        std::string_view settingValue);

    TelemetrySettingOverrides& GetTargetOverrides(
        std::string_view permissionOrProviderName,
        std::string_view eventName);

    static void LogSettingRejection(
        std::string_view settingName,
        std::string_view settingValue,
        std::string_view rejectionReason);

  private:
    TelemetrySettingOverrides _requiredPermissionOverrides;
    TelemetrySettingOverrides _optionalPermissionOverrides;

    // First key is provider name.
    // Second key is empty for provider override, or event name.
    std::map<
        std::string,
        std::map<std::string, TelemetrySettingOverrides, Azure::Device::Utils::String::CaseInsensitiveLessThanPredicate>,
        Azure::Device::Utils::String::CaseInsensitiveLessThanPredicate>
        _providerAndEventSettings;
};

}}}
