// Copyright (c) Microsoft Corporation. All rights reserved.

#include "EventStoreFactory.h"

#include "TelemetryError.h"
#include "azure-device-util/ResultException.h"

#ifdef AZURE_DEVICE_TELEMETRY_USE_STORAGE_SQLITE
    #include "sqlite/SqliteEventStore.h"
#endif

namespace Azure { namespace Device { namespace Telemetry {

#ifdef AZURE_DEVICE_TELEMETRY_USE_STORAGE_SQLITE
std::unique_ptr<IEventStore> EventStoreFactory::CreateSqliteEventStore(
    HeartBeat& heartBeat,
    IEventSerializer& serializer,
    IAsimovEventBufferFactory& asimovEventBufferFactory,
    std::string_view databaseDirectory,
    uint32_t maxSizeInBytes,
    uint32_t preCompressBufferNormalEventCount,
    uint32_t preCompressBufferCriticalEventCount,
    uint32_t preCompressBufferByteLimit)
{
    return std::make_unique<SqliteEventStore>(
        heartBeat,
        serializer,
        asimovEventBufferFactory,
        databaseDirectory,
        maxSizeInBytes,
        preCompressBufferNormalEventCount,
        preCompressBufferCriticalEventCount,
        preCompressBufferByteLimit);
}
#endif

}}}