// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <chrono>

namespace Azure { namespace Device { namespace Telemetry {

class BackoffConfig
{
  public:
    void Reset();
    void Backoff();
    void Pause(uint32_t customBackoffSeconds);

    std::chrono::steady_clock::time_point GetBackoffUntilTime() const
    {
        return _backoffUntilTime;
    }

    void _UseZeroBaseTime() { _useZeroBaseTime = true; }

  private:
    static uint32_t CalculateRandomizedBackoff(
        uint32_t baseBackoffSeconds,
        uint32_t minBackoffSeconds);

    std::chrono::steady_clock::time_point NowTime();

  private:
    uint32_t _currentBackoffAttempts = 0;
    std::chrono::steady_clock::time_point _backoffUntilTime;
    bool _useZeroBaseTime = false;
};

}}}
