// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <cstddef>
#include <vector>
#include <zlib.h>

#include <nonstd/span.hpp>

namespace Azure { namespace Device { namespace Utils {
//
// This class compresses a stream of input data using zlib and produces an
// output buffer. This object should only be used for a single Finish() call,
// and the object should be destroyed.
//
class DeflateStream
{
  public:
    DeflateStream();
    ~DeflateStream();

    void AddData(nonstd::span<const std::byte> data);

    std::vector<std::byte> Finish();

    size_t GetMaxCurrentOutputSize() const;

    static size_t GetCompressorMemoryCost();

  private:
    const static int32_t k_windowBits = 15;
    const static int32_t k_memLevel = 8;

  private:
    z_stream m_zStream;
    std::vector<std::byte> m_outputBytes;
    bool m_finished;
};

}}}
