// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <chrono>
#include <optional>
#include <string>
#include <vector>

namespace Azure { namespace Device { namespace Utils {

class Docker
{
  public:
    struct Container
    {
        std::string ID;
        std::string Name;
        std::string ImageName;
        std::string ImageTag;
        std::string ImageId;
        std::string CreatedTime;
        std::string Status;
        std::string StartedTime;
        std::string FinishedTime;
        std::uint64_t SizeRw;
        std::uint64_t SizeRootFs;
        std::string CpuUsage;
        std::string MemUsage;
    };

    static std::vector<Container> GetAllContainers();
    static std::vector<Container>
    GetContainers(const std::vector<std::string>& containerIds = {});
    static std::optional<Container>
    GetContainer(const std::string_view containerId);
    static bool AddContainerStats(std::vector<Container>& containers);

  private:
    static std::vector<std::string> GetAllContainerIDs();

  private:
    // Container attributes to be collected from 'docker inspect'
    // Query formats MUST match the number and order of elements in this enum
    enum class ContainerAttributes
    {
        ID,
        Name,
        Image,
        ImageID,
        CreatedTime,
        Status,
        StartedTime,
        FinishedTime,
        SizeRw,
        SizeRootFs,
        NUM_VALUES
    };

    // Container stats to be collected from 'docker stats'
    // Query formats MUST match the number and order of elements in this enum
    enum class ContainerStatsFields
    {
        CpuPercentage,
        MemUsage,
        MemPercentage,
        PIDs,
        NUM_VALUES
    };

    /**
     * DOCKER INSPECT QUERY FORMATS
     * Must use the delimiter stated in c_ContainerFormatDelimiter
     *  to separate attributes.
     */

    static constexpr const char* c_ContainerFormatDelimiter = "|";

    static constexpr auto c_DockerInspectFormat =
        "{{.ID}}|{{.Name}}|{{.Config.Image}}|{{.Image}}|{{.Created}}|{{.State.Status}}|{{.State.StartedAt}}|{{.State.FinishedAt}}|{{.SizeRw}}|{{.SizeRootFs}}";

    static constexpr auto c_DockerStatsFormat =
        "{{.CPUPerc}}|{{.MemUsage}}|{{.MemPerc}}|{{.PIDs}}";
};

}}}