// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <fstream>
#include <functional>
#include <map>
#include <string>
#include <string_view>
#include <system_error>

namespace Azure { namespace Device { namespace Utils {

class FileSystem
{
  public:
    static std::error_code
    DeleteAllFilesWithPattern(std::string_view directory, std::string_view pattern);

    static std::error_code CreateDirectoryAndParents(std::string_view directory);

    static std::error_code DoForEachFileWithPattern(
        std::string_view directory,
        std::string_view pattern,
        const std::function<void(const std::filesystem::path&)>& actionOnFile);

    static std::error_code
    WriteUTF8StringToFile(std::string_view fileName, std::string_view data);

    static std::error_code
    ReadUTF8StringFromFile(std::string_view fileName, std::string& data);

    static std::error_code
    ReadFileSize(std::string_view fileName, /*OUT*/ int64_t& size);

    static std::error_code ReadDictionaryFromFile(
        std::string_view fileName,
        std::string_view delimiter,
        std::map<std::string, std::string, std::less<>>& data);

    static bool FileExists(std::string_view fileName);

    static std::filesystem::path GetConfDirectory();
    static std::filesystem::path GetBinaryDirectory();
    static std::filesystem::path GetLibraryDirectory();
    static std::filesystem::path GetLocalStateDirectory();

    class TempOutputFile
    {
      public:
        explicit TempOutputFile(std::filesystem::path filePath)
            : _path(std::move(filePath))
            , _stream(_path.string(), std::ofstream::out)
        {}

        ~TempOutputFile()
        {
            std::error_code ec;
            std::filesystem::remove(_path, ec);
        }

        const std::filesystem::path& GetPath() const { return _path; }
        std::ofstream& GetStream() { return _stream; }

      private:
        std::filesystem::path _path;
        std::ofstream _stream;
    };
};

}}}
