// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <string>
#include <string_view>
#include <variant>
#include <vector>

namespace Azure { namespace Device { namespace Utils {

class ConfigPath
{
    using ComponentVariant = std::variant<std::string, size_t>;

  public:
    ConfigPath() = default;
    ConfigPath(std::string_view periodEmbeddedPath);

    const std::vector<ComponentVariant>& GetComponents() const
    {
        return _components;
    }

    bool operator==(const ConfigPath& other) const
    {
        return _components == other._components;
    }

    ConfigPath CreateWithChildIndex(size_t index) const;
    ConfigPath CreateWithChildName(std::string name) const;

    std::string ToString() const;

  private:
    std::vector<ComponentVariant> _components;
};

class IConfig
{
  public:
    virtual ~IConfig() {}

    enum class ValidateStatus
    {
        Ok,
        PathNotFound,
        TypeError,
    };

    virtual ValidateStatus
    ValidateConfigPathExistsBool(const ConfigPath& key) = 0;
    virtual ValidateStatus ValidateConfigPathExistsInt(const ConfigPath& key) = 0;
    virtual ValidateStatus
    ValidateConfigPathExistsDouble(const ConfigPath& key) = 0;
    virtual ValidateStatus
    ValidateConfigPathExistsString(const ConfigPath& key) = 0;

    virtual bool GetBool(const ConfigPath& key, bool defaultVal) = 0;
    virtual int64_t GetInt(const ConfigPath& key, int64_t defaultVal) = 0;
    virtual double GetDouble(const ConfigPath& key, double defaultVal) = 0;
    virtual std::string
    GetString(const ConfigPath& key, std::string_view defaultVal) = 0;

    virtual size_t GetArraySize(const ConfigPath& key) = 0;
};

}}}
