// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <filesystem>
#include <optional>
#include <string>
#include <string_view>
#include <vector>

#include "Process.h"

namespace Azure { namespace Device { namespace Utils {

class PackageManager
{
  public:
    enum class PackageManagerFamily
    {
        UNKNOWN = 0,
        OTHER = 1,
        RPM = 2,
        DPKG = 3
    };

    struct Package
    {
        std::string Name;
        std::string Version;
        std::string Arch;
    };

    static bool PackageManagerSupported();
    static PackageManagerFamily GetSystemPackageManager();
    static std::optional<Package> GetFileOwner(std::string_view filePath);
    static std::optional<Package> GetProcessOwnerByPid(uint32_t processId);
    static std::vector<Package> GetInstalledPackages();

  private:
    static std::optional<Package> ParsePackage(std::string_view entry);

    static std::vector<Package>
    ParsePackageList(const std::vector<std::string>& list);

    static std::string GetDPKGPackageName(std::string_view filePath);

    static std::optional<Package> GetFileOwnerRPM(std::string_view filePath);
    static std::optional<Package> GetFileOwnerDPKG(std::string_view filePath);

    static std::vector<Package> GetInstalledPackagesRPM();
    static std::vector<Package> GetInstalledPackagesDPKG();

  private:
    // Package attributes to be collected.
    // Query formats MUST match the number and order of elements in this enum
    enum class PackageAttributes
    {
        Name,
        Version,
        Architecture,
        AttributeCount
    };

    /**
     * PACKAGE MANAGER QUERY FORMATS
     * Must use the delimiter stated in c_PackageFormatDelimiter
     *  to separate attributes.
     */

    static constexpr const char* c_PackageFormatDelimiter = " ";

    static constexpr auto c_RPMPackageFormat =
        "%{name} %{version}-%{release} %{arch}\\n";

    static constexpr auto c_DPKGPackageFormat =
        "-f=${Package} ${Version} ${Architecture}\\n";
};

}}}