// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <type_traits>
#include <utility>

namespace Azure { namespace Device { namespace Utils {

template<class LambdaType>
class ScopeExit
{
    ScopeExit(const ScopeExit&) = delete;
    ScopeExit& operator=(const ScopeExit&) = delete;
    ScopeExit& operator=(ScopeExit&&) = delete;

  public:
    ScopeExit(LambdaType&& lambda)
        : _lambda(std::move(lambda)), _shouldCall(true)
    {
        static_assert(
            std::is_void<decltype(lambda())>::value,
            "Only void returning lambdas are allowed");
    }

    ScopeExit(ScopeExit&& other)
        : _lambda(std::move(other._lambda)), _shouldCall(true)
    {
        other._shouldCall = false;
    }

    ~ScopeExit()
    {
        if (_shouldCall)
        {
            _lambda();
        }
    }

    void Cancel() { _shouldCall = false; }

  private:
    LambdaType _lambda;
    bool _shouldCall;
};

template<class LambdaType>
ScopeExit<LambdaType> MakeScopeExit(LambdaType&& lambda)
{
    return ScopeExit<LambdaType>{ std::forward<LambdaType>(lambda) };
}

}}}