// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <cstddef>
#include <iostream>
#include <unistd.h>
#include <vector>

#include <nonstd/span.hpp>

namespace Azure { namespace Device { namespace Utils {

class StreamCopier
{
  public:
    StreamCopier(
        std::istream& inStream,
        std::ostream& out,
        size_t stepSize = sysconf(_SC_PAGE_SIZE));

    void CopyToSpecifiedOffset(size_t desiredOffset);
    void CopyNextBytes(nonstd::span<std::byte> buf);
    void CopyToEnd();

    size_t GetBytesRead() const;

  private:
    std::istream& _inStream;
    std::ostream& _outStream;

    std::vector<std::byte> _tempBuffer;
    size_t _bytesRead = 0;
};

}}}
