// Copyright (c) Microsoft Corporation. All rights reserved.

#pragma once

#include <functional>
#include <memory>
#include <string>
#include <string_view>

#include <azure-device-util/IConfig.h>
#include <azure-device-util/Toml.h>

namespace Azure { namespace Device { namespace Utils {

class TomlConfig : public IConfig
{
  public:
    TomlConfig(std::string_view tomlFilePath);

    // IConfig
    ValidateStatus ValidateConfigPathExistsBool(const ConfigPath& key) override;
    ValidateStatus ValidateConfigPathExistsInt(const ConfigPath& key) override;
    ValidateStatus ValidateConfigPathExistsDouble(const ConfigPath& key) override;
    ValidateStatus ValidateConfigPathExistsString(const ConfigPath& key) override;

    bool GetBool(const ConfigPath& key, bool defaultVal) override;
    int64_t GetInt(const ConfigPath& key, int64_t defaultVal) override;
    double GetDouble(const ConfigPath& key, double defaultVal) override;
    std::string
    GetString(const ConfigPath& key, std::string_view defaultVal) override;

    size_t GetArraySize(const ConfigPath& key) override;

  private:
    const toml::value* FindPath(const ConfigPath& key);

    template<class ReturnType, class DefaultValType>
    ReturnType GetHelper(const ConfigPath& key, DefaultValType defaultVal);

    ValidateStatus ValidateHelper(
        const ConfigPath& key,
        const std::function<bool(const toml::value&)>& typeTest);

  private:
    toml::value _toml;
};

}}}
