// Copyright (c) Microsoft Corporation. All rights reserved.

#include <azure-device-util/ErrorHelper.h>

namespace Azure { namespace Device { namespace Utils {

// Utils library error category
class UtilsErrorCat : public std::error_category
{
    const char* name() const noexcept override { return "UtilsErrorCat"; }
    std::string message(int e) const override
    {
        return "UtilsError: " + std::to_string(e);
    }
};

// Global error category definition
UtilsErrorCat c_utilsErrorCat;

// Required for creating an error code from an error enum
std::error_code make_error_code(UtilsError error)
{
    return std::error_code{ static_cast<int>(error), c_utilsErrorCat };
}

}}}

// Zlib error category
class ZLibErrorCat : public std::error_category
{
    const char* name() const noexcept override { return "ZLibErrorCat"; }
    std::string message(int e) const override
    {
        return "ZLibError: " + std::to_string(e);
    }
};

// Global error category definition
ZLibErrorCat c_zlibErrorCat;

// Required for creating an error code from a ZLIB error int
std::error_code make_zlib_error_code(int zlibError)
{
    return std::error_code{ static_cast<int>(zlibError), c_zlibErrorCat };
}