// Copyright (c) Microsoft Corporation. All rights reserved.

#include <azure-device-util/Hash.h>
#include <openssl/sha.h>

#include "SpookyV2.h"

namespace Azure { namespace Device { namespace Utils {

/*++
    Generates a SHA 256 hash based on an input buffer.

Arguments:
    data - The input data buffer.

Return Value:
    Vector of SHA256_DIGEST_LENGTH containing hash value.
--*/
std::vector<std::byte> Hash::SHA256(nonstd::span<const std::byte> data)
{
    std::vector<std::byte> result(SHA256_DIGEST_LENGTH, std::byte{ 0 });

    ::SHA256(
        const_cast<uint8_t*>(reinterpret_cast<const uint8_t*>(data.data())),
        static_cast<uint32_t>(data.size()),
        reinterpret_cast<uint8_t*>(result.data()));

    return result;
}

/*++
    Generates a Spooky 64 hash based on an input buffer and seed.

Arguments:
    data - The input data buffer.
    seed - The seed to specify for the hasher.

Return Value:
    Spooky hash as uint64
--*/
uint64_t Hash::Spooky64(nonstd::span<const std::byte> data, uint64_t seed)
{
    uint64_t hash1 = seed, hash2 = seed;
    SpookyHash::Hash128(data.data(), data.size(), &hash1, &hash2);
    return hash1;
}

}}}