// Copyright (c) Microsoft Corporation. All rights reserved.

#include <azure-device-util/IConfig.h>
#include <azure-device-util/String.h>

namespace Azure { namespace Device { namespace Utils {

ConfigPath::ConfigPath(std::string_view periodEmbeddedPath)
{
    for (auto& propName : Utils::String::Split(periodEmbeddedPath, ".", false))
    {
        _components.emplace_back(std::move(propName));
    }
}

ConfigPath ConfigPath::CreateWithChildIndex(size_t index) const
{
    ConfigPath child = *this;
    child._components.emplace_back(index);
    return child;
}

ConfigPath ConfigPath::CreateWithChildName(std::string name) const
{
    ConfigPath child = *this;
    child._components.emplace_back(std::move(name));
    return child;
}

std::string ConfigPath::ToString() const
{
    std::stringstream output;
    for (const auto& element : _components)
    {
        if (std::holds_alternative<std::string>(element))
        {
            output << "." << std::get<std::string>(element);
        }
        else
        {
            output << "[" << std::get<size_t>(element) << "]";
        }
    }

    return output.str();
}

}}}
