// Copyright (c) Microsoft Corporation. All rights reserved.

#include <azure-device-util/FileSystem.h>
#include <azure-device-util/Identity.h>
#include <azure-device-util/String.h>

namespace Azure { namespace Device { namespace Utils {

constexpr std::string_view Identity::c_localIdPrefix;

std::string Identity::GetMachineBasedDeviceId(bool prefix)
{
#ifdef AZURE_DEVICE_UTILS_USE_SYSTEMD_BASED_IDENTITY
    constexpr sd_id128_t c_microsoftAppId = SD_ID128_MAKE(
        58, 6e, 0e, d9, 89, c9, 46, a7, 94, e5, de, 33, 76, 44, d3, 08);

    sd_id128_t appSpecificDeviceId = SD_ID128_NULL;
    int result = sd_id128_get_machine_app_specific(
        c_microsoftAppId, &appSpecificDeviceId);
    if (result < 0)
    {
        std::error_code ec{ result, std::system_category() };
        LOG_HR(ec);
        return "";
    }

    std::string deviceId;
    if (prefix)
    {
        deviceId += c_localIdPrefix;
    }

    deviceId += Utils::String::StringFromGuid(appSpecificDeviceId.bytes, false);

    return deviceId;
#else
    //
    // Returning a blank string here will force ADHS to fall back to
    // our PartA JSON file which will persist a unique ID across reboots
    //
    return "";
#endif
}

}}}
