// Copyright (c) Microsoft Corporation. All rights reserved.

#include <azure-device-util/Json.h>

namespace Azure { namespace Device { namespace Utils {

void Json::InsertJsonValueIntoJsonBuilder(
    jsonbuilder::JsonBuilder& jb,
    jsonbuilder::JsonIterator itr,
    std::string_view name,
    const nlohmann::json& j)
{
    switch (j.type())
    {
    case nlohmann::json::value_t::null:
        jb.push_back(itr, name, jsonbuilder::JsonNull);
        break;
    case nlohmann::json::value_t::boolean:
        jb.push_back(itr, name, j.get<bool>());
        break;
    case nlohmann::json::value_t::string:
        jb.push_back(itr, name, j.get<std::string_view>());
        break;
    case nlohmann::json::value_t::number_integer:
        jb.push_back(itr, name, j.get<int64_t>());
        break;
    case nlohmann::json::value_t::number_unsigned:
        jb.push_back(itr, name, j.get<uint64_t>());
        break;
    case nlohmann::json::value_t::number_float:
        jb.push_back(itr, name, j.get<double>());
        break;
    case nlohmann::json::value_t::object: {
        auto objItr = jb.push_back(itr, name, jsonbuilder::JsonObject);
        for (const auto& [key, value] : j.items())
        {
            InsertJsonValueIntoJsonBuilder(jb, objItr, key, value);
        }
        break;
    }
    case nlohmann::json::value_t::array: {
        auto arrItr = jb.push_back(itr, name, jsonbuilder::JsonArray);
        for (const auto& [_, value] : j.items())
        {
            InsertJsonValueIntoJsonBuilder(jb, arrItr, "", value);
        }
        break;
    }
    case nlohmann::json::value_t::discarded: {
        // Just skip any discarded elements
        break;
    }
    default:
        throw std::runtime_error("Unsupported JSON type!");
    }
}
}}}
