// Copyright (c) Microsoft Corporation. All rights reserved.

#include <azure-device-util/StreamCopier.h>

namespace Azure { namespace Device { namespace Utils {

StreamCopier::StreamCopier(std::istream& inStream, std::ostream& out, size_t stepSize)
    : _inStream(inStream), _outStream(out), _tempBuffer(stepSize, std::byte{})
{
    _inStream.exceptions(std::istream::eofbit | std::istream::badbit);
    _outStream.exceptions(std::ostream::badbit);
}

void StreamCopier::CopyToSpecifiedOffset(size_t desiredOffset)
{
    if (desiredOffset < _bytesRead)
    {
        throw std::invalid_argument("StreamCopier::SeekTo cannot seek backwards");
    }

    while (_bytesRead < desiredOffset)
    {
        size_t bytesRemaining = desiredOffset - _bytesRead;
        size_t bytesToRead = std::min(bytesRemaining, _tempBuffer.size());
        CopyNextBytes({ _tempBuffer.data(), bytesToRead });
    }
}

void StreamCopier::CopyToEnd()
{
    // Now that we are copying to the end, eof is acceptable, so removed it
    // from exceptions list and copy until we hit it
    _inStream.exceptions(std::istream::badbit);
    while (!_inStream.eof())
    {
        CopyNextBytes(_tempBuffer);
    }
}

void StreamCopier::CopyNextBytes(nonstd::span<std::byte> buf)
{
    char* bufAsChar = reinterpret_cast<char*>(buf.data());
    _inStream.read(bufAsChar, buf.size());
    size_t count = _inStream.gcount();
    _bytesRead += count;
    _outStream.write(bufAsChar, count);
}

size_t StreamCopier::GetBytesRead() const
{
    return _bytesRead;
}

}}}
